######################## Virtual NC Controller #########################
#
#  Created by camacho  @  Thu Sep 02 15:01:09 2004 Pacific Daylight Time
#  with Post Builder version  3.3.0.
#
########################################################################


set mom_sim_vnc_handler_loaded "[join [split [info script] \\] /]"


set mom_sim_post_builder_version 3.3.0


if { ![info exists sim_mtd_initialized] } {
   set sim_mtd_initialized 0
}


#++++++++++++++++++++++++++++++++++++
# Define the base elements only once!
#++++++++++++++++++++++++++++++++++++
if { $sim_mtd_initialized == 0 } {



set cam_aux_dir  [MOM_ask_env_var UGII_CAM_AUXILIARY_DIR]
set cam_post_dir  [MOM_ask_env_var UGII_CAM_POST_DIR]


set vnc_base_sourced 0


if { !$vnc_base_sourced } {


#=============================================================
proc VNC_ask_shared_library_suffix { } {
#=============================================================
   global tcl_platform

   if [string match "*windows*" $tcl_platform(platform)] {

      set suffix dll

   } else {

      if [string match "*HP-UX*" $tcl_platform(os)] {
         set suffix sl
      } elseif [string match "*AIX*" $tcl_platform(os)] {
         set suffix a
      } else {
         set suffix so
      }
   }

return $suffix
}


set suff [VNC_ask_shared_library_suffix]


if { ![file exists ${cam_aux_dir}mom_source.$suff] } {
   set suff so
}

if { [file exists ${cam_aux_dir}mom_source.$suff] && \
     [file exists ${cam_post_dir}vnc_base_tcl.txt] } {

   MOM_run_user_function ${cam_aux_dir}mom_source.$suff ufusr
   MOM_decrypt_source ${cam_post_dir}vnc_base_tcl.txt

} else {
   set __msg "ERROR in [info script] :\n\
             \nShared library \"mom_source\" is not found in ${cam_aux_dir} or\
             \n\"vnc_base_tcl.txt\" is not found in ${cam_post_dir} directory."

   catch { SIM_mtd_reset }
   MOM_abort $__msg
}


#----------------------------------------------
# Validate correct version of vnc_base sourced.
#----------------------------------------------
if { ![info exists mom_sim_vnc_base_version] || \
      [string compare "3.3.0" $mom_sim_vnc_base_version] > 0 } {

   set __msg "ERROR :\n\
             \nWrong version of \"vnc_base_tcl.txt\" has been detected.\
             \nCorrect version can be found in POSTBUILD directory.\
             \nPlease copy it to your ${cam_post_dir} directory."

   catch { SIM_mtd_reset }
   MOM_abort $__msg
}


} ;# vnc_base sourced


} ;# if { $sim_mtd_initialized == 0 }


#=============================================================
proc VNC_set_nc_definitions {} {
#=============================================================
  global mom_sim_control_out mom_sim_control_in
  global mom_sim_word_separator mom_sim_word_separator_len
  global mom_sim_end_of_block mom_sim_end_of_block_len
  global mom_sim_home_pos
  global mom_sim_address
  global mom_sim_format
  global mom_sim_wcs_offsets
  global mom_sim_nc_register
  global mom_sim_nc_func
  global mom_sim_circular_vector
  global mom_sim_arc_output_mode
  global mom_sim_machine_type
  global mom_sim_num_machine_axes
  global mom_sim_advanced_kinematic_jct
  global mom_sim_4th_axis_plane mom_sim_5th_axis_plane
  global mom_sim_reverse_4th_table mom_sim_reverse_5th_table
  global mom_sim_cycle_mode
  global mom_sim_x_double mom_sim_i_double
  global mom_sim_x_factor mom_sim_y_factor mom_sim_z_factor
  global mom_sim_output_unit
  global mom_sim_cycle_feed_to_addr
  global mom_sim_pivot_distance
  global mom_sim_4th_axis_angle_offset mom_sim_5th_axis_angle_offset
  global mom_sim_4th_axis_direction mom_sim_5th_axis_direction
  global mom_sim_4th_axis_max_limit mom_sim_5th_axis_max_limit
  global mom_sim_4th_axis_min_limit mom_sim_5th_axis_min_limit
  global mom_sim_output_reference_method
  global mom_sim_rapid_feed_rate

   set mom_sim_control_out                            "("
   set mom_sim_control_in                             ")"

   set mom_sim_word_separator                         " "
   set mom_sim_word_separator_len                     "1"

   set mom_sim_end_of_block                           ""
   set mom_sim_end_of_block_len                       "0"

   set mom_sim_home_pos(0)                            "0"
   set mom_sim_home_pos(1)                            "-5"
   set mom_sim_home_pos(2)                            "10"

   set mom_sim_circular_vector                        "Vector - Arc Center to Start"
   set mom_sim_arc_output_mode                        "FULL_CIRCLE"
   set mom_sim_output_unit                            "IN"
   set mom_sim_rapid_feed_rate                        "400"
   set mom_sim_machine_type                           "5_axis_head_table"
   set mom_sim_num_machine_axes                       "5"

   if [info exists mom_sim_advanced_kinematic_jct] {
      unset mom_sim_advanced_kinematic_jct
   }

   set mom_sim_reverse_4th_table                      "0"
   set mom_sim_reverse_5th_table                      "0"
   set mom_sim_4th_axis_plane                         "ZX"
   set mom_sim_5th_axis_plane                         "XY"
   set mom_sim_pivot_distance                         "0.0"
   set mom_sim_4th_axis_angle_offset                  "-90"
   set mom_sim_5th_axis_angle_offset                  "0.0"
   set mom_sim_4th_axis_direction                     "MAGNITUDE_DETERMINES_DIRECTION"
   set mom_sim_5th_axis_direction                     "MAGNITUDE_DETERMINES_DIRECTION"
   set mom_sim_4th_axis_min_limit                     "-120"
   set mom_sim_4th_axis_max_limit                     "120"
   set mom_sim_5th_axis_min_limit                     "-9999"
   set mom_sim_5th_axis_max_limit                     "9999"

   set mom_sim_cycle_feed_to_addr                     "Z"
   set mom_sim_cycle_mode(feed_to)                    "Position"
   set mom_sim_cycle_mode(rapid_to)                   "R - Position"
   set mom_sim_cycle_mode(retract_to)                 "None"
   set mom_sim_cycle_mode(start_block)                "0"

   set mom_sim_x_double                               "1"
   set mom_sim_i_double                               "1"
   set mom_sim_x_factor                               "1"
   set mom_sim_y_factor                               "1"
   set mom_sim_z_factor                               "1"

   set mom_sim_output_reference_method                "GAGE_REF"

   set mom_sim_address(G_cutcom,leader)               "G"
   set mom_sim_address(G_cutcom,leader_len)           "1"
   set mom_sim_address(G_cutcom,trailer)              ""
   set mom_sim_address(G_cutcom,trailer_len)          "0"
   set mom_sim_address(G_cutcom,modal)                "1"
   set mom_sim_address(G_cutcom,format)               "Digit_2"

   set mom_sim_nc_func(CUTCOM_LEFT)                   "G41"
   set mom_sim_nc_func(CUTCOM_RIGHT)                  "G42"
   set mom_sim_nc_func(CUTCOM_OFF)                    "G40"

   set mom_sim_address(G_plane,leader)                "G"
   set mom_sim_address(G_plane,leader_len)            "1"
   set mom_sim_address(G_plane,trailer)               ""
   set mom_sim_address(G_plane,trailer_len)           "0"
   set mom_sim_address(G_plane,modal)                 "1"
   set mom_sim_address(G_plane,format)                "Digit_2"

   set mom_sim_nc_func(PLANE_XY)                      "G17"
   set mom_sim_nc_func(PLANE_ZX)                      "G18"
   set mom_sim_nc_func(PLANE_YZ)                      "G19"

   set mom_sim_address(user_add_g110,leader)          "G"
   set mom_sim_address(user_add_g110,leader_len)      "1"
   set mom_sim_address(user_add_g110,trailer)         ""
   set mom_sim_address(user_add_g110,trailer_len)     "0"
   set mom_sim_address(user_add_g110,modal)           "1"
   set mom_sim_address(user_add_g110,format)          "user_fmt_3_digit"

   set mom_sim_address(G_adjust,leader)               "G"
   set mom_sim_address(G_adjust,leader_len)           "1"
   set mom_sim_address(G_adjust,trailer)              ""
   set mom_sim_address(G_adjust,trailer_len)          "0"
   set mom_sim_address(G_adjust,modal)                "1"
   set mom_sim_address(G_adjust,format)               "Digit_2"

   set mom_sim_nc_func(TL_ADJUST_PLUS)                "G43"
   set mom_sim_nc_func(TL_ADJUST_MINUS)               "G44"
   set mom_sim_nc_func(TL_ADJUST_CANCEL)              "G49"

   set mom_sim_address(G_feed,leader)                 "G"
   set mom_sim_address(G_feed,leader_len)             "1"
   set mom_sim_address(G_feed,trailer)                ""
   set mom_sim_address(G_feed,trailer_len)            "0"
   set mom_sim_address(G_feed,modal)                  "1"
   set mom_sim_address(G_feed,format)                 "Digit_2"

   set mom_sim_nc_func(FEED_IPM)                      "G98"
   set mom_sim_nc_func(FEED_IPR)                      "G99"
   set mom_sim_nc_func(FEED_MMPM)                     "G94"
   set mom_sim_nc_func(FEED_MMPR)                     "G95"
   set mom_sim_nc_func(FEED_FRN)                      "G93"

   set mom_sim_address(G_spin,leader)                 "G"
   set mom_sim_address(G_spin,leader_len)             "1"
   set mom_sim_address(G_spin,trailer)                ""
   set mom_sim_address(G_spin,trailer_len)            "0"
   set mom_sim_address(G_spin,modal)                  "1"
   set mom_sim_address(G_spin,format)                 "Digit_2"

   set mom_sim_nc_func(SPEED_SFM)                     "G96"
   set mom_sim_nc_func(SPEED_SMM)                     "G96"
   set mom_sim_nc_func(SPEED_RPM)                     "G97"

   set mom_sim_address(G_return,leader)               "G"
   set mom_sim_address(G_return,leader_len)           "1"
   set mom_sim_address(G_return,trailer)              ""
   set mom_sim_address(G_return,trailer_len)          "0"
   set mom_sim_address(G_return,modal)                "1"
   set mom_sim_address(G_return,format)               "Digit_2"

   set mom_sim_nc_func(CYCLE_RETURN_AUTO)             "G98"
   set mom_sim_nc_func(CYCLE_RETURN_MANUAL)           "G99"

   set mom_sim_address(G_motion,leader)               "G"
   set mom_sim_address(G_motion,leader_len)           "1"
   set mom_sim_address(G_motion,trailer)              ""
   set mom_sim_address(G_motion,trailer_len)          "0"
   set mom_sim_address(G_motion,modal)                "1"
   set mom_sim_address(G_motion,format)               "String"

   set mom_sim_nc_func(MOTION_RAPID)                  "G0"
   set mom_sim_nc_func(MOTION_LINEAR)                 "G1"
   set mom_sim_nc_func(MOTION_CIRCULAR_CLW)           "G2"
   set mom_sim_nc_func(MOTION_CIRCULAR_CCLW)          "G3"
   set mom_sim_nc_func(CYCLE_DRILL)                   "G81"
   set mom_sim_nc_func(CYCLE_DRILL_DWELL)             "G82"
   set mom_sim_nc_func(CYCLE_DRILL_DEEP)              "G83"
   set mom_sim_nc_func(CYCLE_DRILL_BREAK_CHIP)        "G73"
   set mom_sim_nc_func(CYCLE_TAP)                     "G84"
   set mom_sim_nc_func(CYCLE_BORE)                    "G85"
   set mom_sim_nc_func(CYCLE_BORE_DRAG)               "G86"
   set mom_sim_nc_func(CYCLE_BORE_NO_DRAG)            "G76"
   set mom_sim_nc_func(CYCLE_BORE_BACK)               "G87"
   set mom_sim_nc_func(CYCLE_BORE_MANUAL)             "G88"
   set mom_sim_nc_func(CYCLE_BORE_DWELL)              "G89"
   set mom_sim_nc_func(CYCLE_BORE_MANUAL_DWELL)       "G89"
   set mom_sim_nc_func(CYCLE_START)                   "G79"
   set mom_sim_nc_func(CYCLE_OFF)                     "G80"

   set mom_sim_address(G_mode,leader)                 "G"
   set mom_sim_address(G_mode,leader_len)             "1"
   set mom_sim_address(G_mode,trailer)                ""
   set mom_sim_address(G_mode,trailer_len)            "0"
   set mom_sim_address(G_mode,modal)                  "1"
   set mom_sim_address(G_mode,format)                 "Digit_2"

   set mom_sim_nc_func(INPUT_ABS)                     "G90"
   set mom_sim_nc_func(INPUT_INC)                     "G91"

   set mom_sim_address(G,leader)                      "G"
   set mom_sim_address(G,leader_len)                  "1"
   set mom_sim_address(G,trailer)                     ""
   set mom_sim_address(G,trailer_len)                 "0"
   set mom_sim_address(G,modal)                       "1"
   set mom_sim_address(G,format)                      "Digit_2"

   set mom_sim_nc_func(DELAY_SEC)                     "G04"
   set mom_sim_nc_func(DELAY_REV)                     "G04"
   set mom_sim_nc_func(WORK_CS_RESET)                 "G50"
   set mom_sim_nc_func(RETURN_HOME)                   "G28"
   set mom_sim_nc_func(UNIT_IN)                       "G70"
   set mom_sim_nc_func(UNIT_MM)                       "G71"

   set mom_sim_address(order,leader)                  "P"
   set mom_sim_address(order,leader_len)              "1"
   set mom_sim_address(order,trailer)                 ""
   set mom_sim_address(order,trailer_len)             "0"
   set mom_sim_address(order,modal)                   "1"
   set mom_sim_address(order,format)                  "Digit_2"

   set mom_sim_address(knot,leader)                   "K"
   set mom_sim_address(knot,leader_len)               "1"
   set mom_sim_address(knot,trailer)                  ""
   set mom_sim_address(knot,trailer_len)              "0"
   set mom_sim_address(knot,modal)                    "1"
   set mom_sim_address(knot,format)                   "Coordinate"

   set mom_sim_address(X,leader)                      "X"
   set mom_sim_address(X,leader_len)                  "1"
   set mom_sim_address(X,trailer)                     ""
   set mom_sim_address(X,trailer_len)                 "0"
   set mom_sim_address(X,modal)                       "1"
   set mom_sim_address(X,format)                      "Coordinate"

   set mom_sim_address(U,leader)                      "U"
   set mom_sim_address(U,leader_len)                  "1"
   set mom_sim_address(U,trailer)                     ""
   set mom_sim_address(U,trailer_len)                 "0"
   set mom_sim_address(U,modal)                       "1"
   set mom_sim_address(U,format)                      "AbsCoord"

   set mom_sim_address(Y,leader)                      "Y"
   set mom_sim_address(Y,leader_len)                  "1"
   set mom_sim_address(Y,trailer)                     ""
   set mom_sim_address(Y,trailer_len)                 "0"
   set mom_sim_address(Y,modal)                       "1"
   set mom_sim_address(Y,format)                      "Coordinate"

   set mom_sim_address(Y_zero,leader)                 "Y"
   set mom_sim_address(Y_zero,leader_len)             "1"
   set mom_sim_address(Y_zero,trailer)                ""
   set mom_sim_address(Y_zero,trailer_len)            "0"
   set mom_sim_address(Y_zero,modal)                  "1"
   set mom_sim_address(Y_zero,format)                 "Coordinate"

   set mom_sim_address(V,leader)                      "V"
   set mom_sim_address(V,leader_len)                  "1"
   set mom_sim_address(V,trailer)                     ""
   set mom_sim_address(V,trailer_len)                 "0"
   set mom_sim_address(V,modal)                       "1"
   set mom_sim_address(V,format)                      "AbsCoord"

   set mom_sim_address(Z,leader)                      "Z"
   set mom_sim_address(Z,leader_len)                  "1"
   set mom_sim_address(Z,trailer)                     ""
   set mom_sim_address(Z,trailer_len)                 "0"
   set mom_sim_address(Z,modal)                       "1"
   set mom_sim_address(Z,format)                      "Coordinate"

   set mom_sim_address(W,leader)                      "W"
   set mom_sim_address(W,leader_len)                  "1"
   set mom_sim_address(W,trailer)                     ""
   set mom_sim_address(W,trailer_len)                 "0"
   set mom_sim_address(W,modal)                       "1"
   set mom_sim_address(W,format)                      "AbsCoord"

   set mom_sim_address(fourth_axis,leader)            "B"
   set mom_sim_address(fourth_axis,leader_len)        "1"
   set mom_sim_address(fourth_axis,trailer)           ""
   set mom_sim_address(fourth_axis,trailer_len)       "0"
   set mom_sim_address(fourth_axis,modal)             "1"
   set mom_sim_address(fourth_axis,format)            "Rotary"

   set mom_sim_address(fifth_axis,leader)             "C"
   set mom_sim_address(fifth_axis,leader_len)         "1"
   set mom_sim_address(fifth_axis,trailer)            ""
   set mom_sim_address(fifth_axis,trailer_len)        "0"
   set mom_sim_address(fifth_axis,modal)              "1"
   set mom_sim_address(fifth_axis,format)             "Rotary"

   set mom_sim_address(J,leader)                      "J"
   set mom_sim_address(J,leader_len)                  "1"
   set mom_sim_address(J,trailer)                     ""
   set mom_sim_address(J,trailer_len)                 "0"
   set mom_sim_address(J,modal)                       "1"
   set mom_sim_address(J,format)                      "Coordinate"

   set mom_sim_address(K,leader)                      "K"
   set mom_sim_address(K,leader_len)                  "1"
   set mom_sim_address(K,trailer)                     ""
   set mom_sim_address(K,trailer_len)                 "0"
   set mom_sim_address(K,modal)                       "1"
   set mom_sim_address(K,format)                      "Coordinate"

   set mom_sim_address(R,leader)                      "R"
   set mom_sim_address(R,leader_len)                  "1"
   set mom_sim_address(R,trailer)                     ""
   set mom_sim_address(R,trailer_len)                 "0"
   set mom_sim_address(R,modal)                       "1"
   set mom_sim_address(R,format)                      "Coordinate"

   set mom_sim_address(F,leader)                      "F"
   set mom_sim_address(F,leader_len)                  "1"
   set mom_sim_address(F,trailer)                     ""
   set mom_sim_address(F,trailer_len)                 "0"
   set mom_sim_address(F,modal)                       "1"
   set mom_sim_address(F,format)                      "Feed"

   set mom_sim_address(S,leader)                      "S"
   set mom_sim_address(S,leader_len)                  "1"
   set mom_sim_address(S,trailer)                     ""
   set mom_sim_address(S,trailer_len)                 "0"
   set mom_sim_address(S,modal)                       "1"
   set mom_sim_address(S,format)                      "Digit_5"

   set mom_sim_address(T,leader)                      "T"
   set mom_sim_address(T,leader_len)                  "1"
   set mom_sim_address(T,trailer)                     ""
   set mom_sim_address(T,trailer_len)                 "0"
   set mom_sim_address(T,modal)                       "1"
   set mom_sim_address(T,format)                      "Tcode"

   set mom_sim_address(D,leader)                      "D"
   set mom_sim_address(D,leader_len)                  "1"
   set mom_sim_address(D,trailer)                     ""
   set mom_sim_address(D,trailer_len)                 "0"
   set mom_sim_address(D,modal)                       "1"
   set mom_sim_address(D,format)                      "Digit_2"

   set mom_sim_address(H,leader)                      "H"
   set mom_sim_address(H,leader_len)                  "1"
   set mom_sim_address(H,trailer)                     ""
   set mom_sim_address(H,trailer_len)                 "0"
   set mom_sim_address(H,modal)                       "1"
   set mom_sim_address(H,format)                      "Hcode"

   set mom_sim_address(dwell,leader)                  "P"
   set mom_sim_address(dwell,leader_len)              "1"
   set mom_sim_address(dwell,trailer)                 ""
   set mom_sim_address(dwell,trailer_len)             "0"
   set mom_sim_address(dwell,modal)                   "1"
   set mom_sim_address(dwell,format)                  "Dwell_sec"

   set mom_sim_address(cycle_dwell,leader)            "P"
   set mom_sim_address(cycle_dwell,leader_len)        "1"
   set mom_sim_address(cycle_dwell,trailer)           ""
   set mom_sim_address(cycle_dwell,trailer_len)       "0"
   set mom_sim_address(cycle_dwell,modal)             "1"
   set mom_sim_address(cycle_dwell,format)            "Dwell_sec"

   set mom_sim_address(P_cutcom,leader)               "P"
   set mom_sim_address(P_cutcom,leader_len)           "1"
   set mom_sim_address(P_cutcom,trailer)              ""
   set mom_sim_address(P_cutcom,trailer_len)          "0"
   set mom_sim_address(P_cutcom,modal)                "1"
   set mom_sim_address(P_cutcom,format)               "Dwell_sec"

   set mom_sim_address(K_cycle,leader)                "K"
   set mom_sim_address(K_cycle,leader_len)            "1"
   set mom_sim_address(K_cycle,trailer)               ""
   set mom_sim_address(K_cycle,trailer_len)           "0"
   set mom_sim_address(K_cycle,modal)                 "1"
   set mom_sim_address(K_cycle,format)                "Coordinate"

   set mom_sim_address(cycle_step1,leader)            "I"
   set mom_sim_address(cycle_step1,leader_len)        "1"
   set mom_sim_address(cycle_step1,trailer)           ""
   set mom_sim_address(cycle_step1,trailer_len)       "0"
   set mom_sim_address(cycle_step1,modal)             "1"
   set mom_sim_address(cycle_step1,format)            "Coordinate"

   set mom_sim_address(cycle_step,leader)             "Q"
   set mom_sim_address(cycle_step,leader_len)         "1"
   set mom_sim_address(cycle_step,trailer)            ""
   set mom_sim_address(cycle_step,trailer_len)        "0"
   set mom_sim_address(cycle_step,modal)              "1"
   set mom_sim_address(cycle_step,format)             "Coordinate"

   set mom_sim_address(Q_cutcom,leader)               "Q"
   set mom_sim_address(Q_cutcom,leader_len)           "1"
   set mom_sim_address(Q_cutcom,trailer)              ""
   set mom_sim_address(Q_cutcom,trailer_len)          "0"
   set mom_sim_address(Q_cutcom,modal)                "1"
   set mom_sim_address(Q_cutcom,format)               "Coordinate"

   set mom_sim_address(E,leader)                      "E"
   set mom_sim_address(E,leader_len)                  "1"
   set mom_sim_address(E,trailer)                     ""
   set mom_sim_address(E,trailer_len)                 "0"
   set mom_sim_address(E,modal)                       "1"
   set mom_sim_address(E,format)                      "Coordinate"

   set mom_sim_address(M_spindle,leader)              "M"
   set mom_sim_address(M_spindle,leader_len)          "1"
   set mom_sim_address(M_spindle,trailer)             ""
   set mom_sim_address(M_spindle,trailer_len)         "0"
   set mom_sim_address(M_spindle,modal)               "1"
   set mom_sim_address(M_spindle,format)              "Digit_4"

   set mom_sim_nc_func(SPINDLE_CLW)                   "M203"
   set mom_sim_nc_func(SPINDLE_CCLW)                  "M4"
   set mom_sim_nc_func(SPINDLE_OFF)                   "M5"

   set mom_sim_address(M_range,leader)                "M"
   set mom_sim_address(M_range,leader_len)            "1"
   set mom_sim_address(M_range,trailer)               ""
   set mom_sim_address(M_range,trailer_len)           "0"
   set mom_sim_address(M_range,modal)                 "1"
   set mom_sim_address(M_range,format)                "Digit_2"

   set mom_sim_address(M_coolant,leader)              "M"
   set mom_sim_address(M_coolant,leader_len)          "1"
   set mom_sim_address(M_coolant,trailer)             ""
   set mom_sim_address(M_coolant,trailer_len)         "0"
   set mom_sim_address(M_coolant,modal)               "1"
   set mom_sim_address(M_coolant,format)              "Digit_2"

   set mom_sim_address(M,leader)                      "M"
   set mom_sim_address(M,leader_len)                  "1"
   set mom_sim_address(M,trailer)                     ""
   set mom_sim_address(M,trailer_len)                 "0"
   set mom_sim_address(M,modal)                       "1"
   set mom_sim_address(M,format)                      "digit_3"

   set mom_sim_nc_func(PROG_STOP)                     "M0"
   set mom_sim_nc_func(PROG_OPSTOP)                   "M1"
   set mom_sim_nc_func(PROG_END)                      "M2"
   set mom_sim_nc_func(TOOL_CHANGE)                   "M999"
   set mom_sim_nc_func(PROG_STOP_REWIND)              "M30"

   set mom_sim_address(cycle_orient,leader)           "Q"
   set mom_sim_address(cycle_orient,leader_len)       "1"
   set mom_sim_address(cycle_orient,trailer)          ""
   set mom_sim_address(cycle_orient,trailer_len)      "0"
   set mom_sim_address(cycle_orient,modal)            "1"
   set mom_sim_address(cycle_orient,format)           "Coordinate"

   set mom_sim_address(I,leader)                      "I"
   set mom_sim_address(I,leader_len)                  "1"
   set mom_sim_address(I,trailer)                     ""
   set mom_sim_address(I,trailer_len)                 "0"
   set mom_sim_address(I,modal)                       "1"
   set mom_sim_address(I,format)                      "Coordinate"

   set mom_sim_address(N,leader)                      "N"
   set mom_sim_address(N,leader_len)                  "1"
   set mom_sim_address(N,trailer)                     ""
   set mom_sim_address(N,trailer_len)                 "0"
   set mom_sim_address(N,modal)                       "1"
   set mom_sim_address(N,format)                      "Digit_4"

   set mom_sim_address(user_add_m202,leader)          "M"
   set mom_sim_address(user_add_m202,leader_len)      "1"
   set mom_sim_address(user_add_m202,trailer)         ""
   set mom_sim_address(user_add_m202,trailer_len)     "0"
   set mom_sim_address(user_add_m202,modal)           "1"
   set mom_sim_address(user_add_m202,format)          "user_fmt_3_digit"

   set mom_sim_address(user_add_m305,leader)          "M"
   set mom_sim_address(user_add_m305,leader_len)      "1"
   set mom_sim_address(user_add_m305,trailer)         ""
   set mom_sim_address(user_add_m305,trailer_len)     "0"
   set mom_sim_address(user_add_m305,modal)           "1"
   set mom_sim_address(user_add_m305,format)          "user_fmt_3_digit"

   set mom_sim_address(user_add_m306,leader)          "M"
   set mom_sim_address(user_add_m306,leader_len)      "1"
   set mom_sim_address(user_add_m306,trailer)         ""
   set mom_sim_address(user_add_m306,trailer_len)     "0"
   set mom_sim_address(user_add_m306,modal)           "1"
   set mom_sim_address(user_add_m306,format)          "user_fmt_3_digit"

   set mom_sim_address(b_next_tool,leader)            "B"
   set mom_sim_address(b_next_tool,leader_len)        "1"
   set mom_sim_address(b_next_tool,trailer)           ""
   set mom_sim_address(b_next_tool,trailer_len)       "0"
   set mom_sim_address(b_next_tool,modal)             "1"
   set mom_sim_address(b_next_tool,format)            "bcode"

   set mom_sim_address(Z_inc,leader)                  "Z"
   set mom_sim_address(Z_inc,leader_len)              "1"
   set mom_sim_address(Z_inc,trailer)                 ""
   set mom_sim_address(Z_inc,trailer_len)             "0"
   set mom_sim_address(Z_inc,modal)                   "1"
   set mom_sim_address(Z_inc,format)                  "Coordinate"

   set mom_sim_address(Text,leader)                   ""
   set mom_sim_address(Text,leader_len)               "0"
   set mom_sim_address(Text,trailer)                  ""
   set mom_sim_address(Text,trailer_len)              "0"
   set mom_sim_address(Text,modal)                    "1"
   set mom_sim_address(Text,format)                   "String"

   set mom_sim_format(AbsCoord,type)                  "Numeral"
   set mom_sim_format(AbsCoord,plus)                  "0"
   set mom_sim_format(AbsCoord,lead_zero)             "0"
   set mom_sim_format(AbsCoord,integer)               "5"
   set mom_sim_format(AbsCoord,decimal)               "1"
   set mom_sim_format(AbsCoord,fraction)              "4"
   set mom_sim_format(AbsCoord,trail_zero)            "0"

   set mom_sim_format(Coordinate,type)                "Numeral"
   set mom_sim_format(Coordinate,plus)                "0"
   set mom_sim_format(Coordinate,lead_zero)           "0"
   set mom_sim_format(Coordinate,integer)             "4"
   set mom_sim_format(Coordinate,decimal)             "1"
   set mom_sim_format(Coordinate,fraction)            "4"
   set mom_sim_format(Coordinate,trail_zero)          "0"

   set mom_sim_format(Digit_2,type)                   "Numeral"
   set mom_sim_format(Digit_2,plus)                   "0"
   set mom_sim_format(Digit_2,lead_zero)              "1"
   set mom_sim_format(Digit_2,integer)                "2"
   set mom_sim_format(Digit_2,decimal)                "0"
   set mom_sim_format(Digit_2,fraction)               "0"
   set mom_sim_format(Digit_2,trail_zero)             "1"

   set mom_sim_format(Digit_4,type)                   "Numeral"
   set mom_sim_format(Digit_4,plus)                   "0"
   set mom_sim_format(Digit_4,lead_zero)              "0"
   set mom_sim_format(Digit_4,integer)                "4"
   set mom_sim_format(Digit_4,decimal)                "0"
   set mom_sim_format(Digit_4,fraction)               "0"
   set mom_sim_format(Digit_4,trail_zero)             "1"

   set mom_sim_format(Digit_5,type)                   "Numeral"
   set mom_sim_format(Digit_5,plus)                   "0"
   set mom_sim_format(Digit_5,lead_zero)              "0"
   set mom_sim_format(Digit_5,integer)                "5"
   set mom_sim_format(Digit_5,decimal)                "0"
   set mom_sim_format(Digit_5,fraction)               "0"
   set mom_sim_format(Digit_5,trail_zero)             "1"

   set mom_sim_format(Dwell_SECONDS,type)             "Numeral"
   set mom_sim_format(Dwell_SECONDS,plus)             "0"
   set mom_sim_format(Dwell_SECONDS,lead_zero)        "0"
   set mom_sim_format(Dwell_SECONDS,integer)          "5"
   set mom_sim_format(Dwell_SECONDS,decimal)          "1"
   set mom_sim_format(Dwell_SECONDS,fraction)         "3"
   set mom_sim_format(Dwell_SECONDS,trail_zero)       "0"

   set mom_sim_format(Dwell_sec,type)                 "Numeral"
   set mom_sim_format(Dwell_sec,plus)                 "0"
   set mom_sim_format(Dwell_sec,lead_zero)            "0"
   set mom_sim_format(Dwell_sec,integer)              "5"
   set mom_sim_format(Dwell_sec,decimal)              "1"
   set mom_sim_format(Dwell_sec,fraction)             "3"
   set mom_sim_format(Dwell_sec,trail_zero)           "0"

   set mom_sim_format(EventNum,type)                  "Numeral"
   set mom_sim_format(EventNum,plus)                  "1"
   set mom_sim_format(EventNum,lead_zero)             "1"
   set mom_sim_format(EventNum,integer)               "3"
   set mom_sim_format(EventNum,decimal)               "0"
   set mom_sim_format(EventNum,fraction)              "0"
   set mom_sim_format(EventNum,trail_zero)            "1"

   set mom_sim_format(Feed,type)                      "Numeral"
   set mom_sim_format(Feed,plus)                      "0"
   set mom_sim_format(Feed,lead_zero)                 "0"
   set mom_sim_format(Feed,integer)                   "7"
   set mom_sim_format(Feed,decimal)                   "1"
   set mom_sim_format(Feed,fraction)                  "2"
   set mom_sim_format(Feed,trail_zero)                "0"

   set mom_sim_format(Feed_DPM,type)                  "Numeral"
   set mom_sim_format(Feed_DPM,plus)                  "0"
   set mom_sim_format(Feed_DPM,lead_zero)             "0"
   set mom_sim_format(Feed_DPM,integer)               "5"
   set mom_sim_format(Feed_DPM,decimal)               "1"
   set mom_sim_format(Feed_DPM,fraction)              "2"
   set mom_sim_format(Feed_DPM,trail_zero)            "0"

   set mom_sim_format(Feed_FRN,type)                  "Numeral"
   set mom_sim_format(Feed_FRN,plus)                  "0"
   set mom_sim_format(Feed_FRN,lead_zero)             "0"
   set mom_sim_format(Feed_FRN,integer)               "5"
   set mom_sim_format(Feed_FRN,decimal)               "1"
   set mom_sim_format(Feed_FRN,fraction)              "3"
   set mom_sim_format(Feed_FRN,trail_zero)            "0"

   set mom_sim_format(Feed_INV,type)                  "Numeral"
   set mom_sim_format(Feed_INV,plus)                  "0"
   set mom_sim_format(Feed_INV,lead_zero)             "0"
   set mom_sim_format(Feed_INV,integer)               "5"
   set mom_sim_format(Feed_INV,decimal)               "1"
   set mom_sim_format(Feed_INV,fraction)              "3"
   set mom_sim_format(Feed_INV,trail_zero)            "0"

   set mom_sim_format(Feed_IPM,type)                  "Numeral"
   set mom_sim_format(Feed_IPM,plus)                  "0"
   set mom_sim_format(Feed_IPM,lead_zero)             "0"
   set mom_sim_format(Feed_IPM,integer)               "4"
   set mom_sim_format(Feed_IPM,decimal)               "1"
   set mom_sim_format(Feed_IPM,fraction)              "1"
   set mom_sim_format(Feed_IPM,trail_zero)            "0"

   set mom_sim_format(Feed_IPR,type)                  "Numeral"
   set mom_sim_format(Feed_IPR,plus)                  "0"
   set mom_sim_format(Feed_IPR,lead_zero)             "0"
   set mom_sim_format(Feed_IPR,integer)               "1"
   set mom_sim_format(Feed_IPR,decimal)               "1"
   set mom_sim_format(Feed_IPR,fraction)              "4"
   set mom_sim_format(Feed_IPR,trail_zero)            "0"

   set mom_sim_format(Hcode,type)                     "Numeral"
   set mom_sim_format(Hcode,plus)                     "0"
   set mom_sim_format(Hcode,lead_zero)                "1"
   set mom_sim_format(Hcode,integer)                  "2"
   set mom_sim_format(Hcode,decimal)                  "0"
   set mom_sim_format(Hcode,fraction)                 "0"
   set mom_sim_format(Hcode,trail_zero)               "1"

   set mom_sim_format(Rev,type)                       "Numeral"
   set mom_sim_format(Rev,plus)                       "0"
   set mom_sim_format(Rev,lead_zero)                  "0"
   set mom_sim_format(Rev,integer)                    "4"
   set mom_sim_format(Rev,decimal)                    "0"
   set mom_sim_format(Rev,fraction)                   "0"
   set mom_sim_format(Rev,trail_zero)                 "1"

   set mom_sim_format(Rotary,type)                    "Numeral"
   set mom_sim_format(Rotary,plus)                    "0"
   set mom_sim_format(Rotary,lead_zero)               "0"
   set mom_sim_format(Rotary,integer)                 "5"
   set mom_sim_format(Rotary,decimal)                 "1"
   set mom_sim_format(Rotary,fraction)                "3"
   set mom_sim_format(Rotary,trail_zero)              "0"

   set mom_sim_format(String,type)                    "Text String"
   set mom_sim_format(String,plus)                    "0"
   set mom_sim_format(String,lead_zero)               "0"
   set mom_sim_format(String,integer)                 ""
   set mom_sim_format(String,decimal)                 "0"
   set mom_sim_format(String,fraction)                ""
   set mom_sim_format(String,trail_zero)              "0"

   set mom_sim_format(Tcode,type)                     "Numeral"
   set mom_sim_format(Tcode,plus)                     "0"
   set mom_sim_format(Tcode,lead_zero)                "1"
   set mom_sim_format(Tcode,integer)                  "4"
   set mom_sim_format(Tcode,decimal)                  "1"
   set mom_sim_format(Tcode,fraction)                 "1"
   set mom_sim_format(Tcode,trail_zero)               "1"

   set mom_sim_format(Zero_int,type)                  "Numeral"
   set mom_sim_format(Zero_int,plus)                  "0"
   set mom_sim_format(Zero_int,lead_zero)             "1"
   set mom_sim_format(Zero_int,integer)               "1"
   set mom_sim_format(Zero_int,decimal)               "0"
   set mom_sim_format(Zero_int,fraction)              "0"
   set mom_sim_format(Zero_int,trail_zero)            "0"

   set mom_sim_format(Zero_real,type)                 "Numeral"
   set mom_sim_format(Zero_real,plus)                 "0"
   set mom_sim_format(Zero_real,lead_zero)            "1"
   set mom_sim_format(Zero_real,integer)              "1"
   set mom_sim_format(Zero_real,decimal)              "1"
   set mom_sim_format(Zero_real,fraction)             "1"
   set mom_sim_format(Zero_real,trail_zero)           "1"

   set mom_sim_format(bcode,type)                     "Numeral"
   set mom_sim_format(bcode,plus)                     "0"
   set mom_sim_format(bcode,lead_zero)                "1"
   set mom_sim_format(bcode,integer)                  "2"
   set mom_sim_format(bcode,decimal)                  "0"
   set mom_sim_format(bcode,fraction)                 "0"
   set mom_sim_format(bcode,trail_zero)               "1"

   set mom_sim_format(digit_3,type)                   "Numeral"
   set mom_sim_format(digit_3,plus)                   "0"
   set mom_sim_format(digit_3,lead_zero)              "0"
   set mom_sim_format(digit_3,integer)                "3"
   set mom_sim_format(digit_3,decimal)                "0"
   set mom_sim_format(digit_3,fraction)               "0"
   set mom_sim_format(digit_3,trail_zero)             "1"

   set mom_sim_format(user_fmt_3_digit,type)          "Numeral"
   set mom_sim_format(user_fmt_3_digit,plus)          "0"
   set mom_sim_format(user_fmt_3_digit,lead_zero)     "1"
   set mom_sim_format(user_fmt_3_digit,integer)       "3"
   set mom_sim_format(user_fmt_3_digit,decimal)       "0"
   set mom_sim_format(user_fmt_3_digit,fraction)      "0"
   set mom_sim_format(user_fmt_3_digit,trail_zero)    "0"

}


#=============================================================
proc PB_CMD_vnc__G_adjust_code { } {
#=============================================================
  global mom_sim_nc_register
  global mom_sim_nc_func


   set codes_list [list]
    lappend codes_list $mom_sim_nc_func(TL_ADJUST_PLUS)
    lappend codes_list $mom_sim_nc_func(TL_ADJUST_MINUS)
    lappend codes_list $mom_sim_nc_func(TL_ADJUST_CANCEL)


   global mom_sim_o_buffer
   foreach code $codes_list {

      if { [VNC_parse_nc_block mom_sim_o_buffer $code] == 1 } {

         if [string match "$mom_sim_nc_func(TL_ADJUST_PLUS)" $code] {

           # When there's been a tool change, reset ref junction.
           # We may need to retrieve the actual tool length, if any, from H register.

            set mom_sim_nc_register(TL_ADJUST) PLUS

         } elseif [string match "$mom_sim_nc_func(TL_ADJUST_MINUS)"  $code] {

            set mom_sim_nc_register(TL_ADJUST) MINUS

         } elseif [string match "$mom_sim_nc_func(TL_ADJUST_CANCEL)" $code] {

            set mom_sim_nc_register(TL_ADJUST) OFF
         }

         break
      }
   }
}


#=============================================================
proc PB_CMD_vnc__G_feed_code { } {
#=============================================================
  global mom_sim_nc_register
  global mom_sim_nc_func


   set codes_list [list]
    lappend codes_list $mom_sim_nc_func(FEED_IPM)
    lappend codes_list $mom_sim_nc_func(FEED_IPR)
    lappend codes_list $mom_sim_nc_func(FEED_MMPM)
    lappend codes_list $mom_sim_nc_func(FEED_MMPR)
    lappend codes_list $mom_sim_nc_func(FEED_FRN)


   global mom_sim_o_buffer
   foreach code $codes_list {

      if { [VNC_parse_nc_block mom_sim_o_buffer $code] == 1 } {

         if [string match "$mom_sim_nc_func(FEED_IPM)" $code] {

            set mom_sim_nc_register(FEED_MODE) INCH_PER_MIN

         } elseif [string match "$mom_sim_nc_func(FEED_IPR)"  $code] {

            set mom_sim_nc_register(FEED_MODE) INCH_PER_REV

         } elseif [string match "$mom_sim_nc_func(FEED_MMPM)"  $code] {

            set mom_sim_nc_register(FEED_MODE) MM_PER_MIN

         } elseif [string match "$mom_sim_nc_func(FEED_MMPR)"  $code] {

            set mom_sim_nc_register(FEED_MODE) MM_PER_REV

         } elseif [string match "$mom_sim_nc_func(FEED_FRN)"  $code] {

            set mom_sim_nc_register(FEED_MODE) MM_PER_100REV
         }

         break
      }
   }
}


#=============================================================
proc PB_CMD_vnc__G_misc_code { } {
#=============================================================
  global mom_sim_pos
  global mom_sim_nc_register
  global mom_sim_nc_func
  global mom_sim_wcs_offsets


   set codes_list [list]
    lappend codes_list $mom_sim_nc_func(DELAY_SEC)
    lappend codes_list $mom_sim_nc_func(DELAY_REV)
    lappend codes_list $mom_sim_nc_func(UNIT_IN)
    lappend codes_list $mom_sim_nc_func(UNIT_MM)
    lappend codes_list $mom_sim_nc_func(RETURN_HOME)
    lappend codes_list $mom_sim_nc_func(RETURN_HOME_P)
    lappend codes_list $mom_sim_nc_func(FROM_HOME)
    lappend codes_list $mom_sim_nc_func(MACH_CS_MOVE)
    lappend codes_list $mom_sim_nc_func(WORK_CS_1)
    lappend codes_list $mom_sim_nc_func(WORK_CS_2)
    lappend codes_list $mom_sim_nc_func(WORK_CS_3)
    lappend codes_list $mom_sim_nc_func(WORK_CS_4)
    lappend codes_list $mom_sim_nc_func(WORK_CS_5)
    lappend codes_list $mom_sim_nc_func(WORK_CS_6)
    lappend codes_list $mom_sim_nc_func(LOCAL_CS_SET)
    lappend codes_list $mom_sim_nc_func(WORK_CS_RESET)


   global mom_sim_o_buffer
   foreach code $codes_list {

      if { [VNC_parse_nc_block mom_sim_o_buffer $code] == 1 } {

         if [string match "$mom_sim_nc_func(DELAY_SEC)" $code] {

         } elseif [string match "$mom_sim_nc_func(DELAY_REV)" $code] {

         } elseif [string match "$mom_sim_nc_func(UNIT_IN)" $code] {

            set mom_sim_nc_register(UNIT) IN
            SIM_set_mtd_units "INCH" 

         } elseif [string match "$mom_sim_nc_func(UNIT_MM)" $code] {

            set mom_sim_nc_register(UNIT) MM
            SIM_set_mtd_units "MM" 

         } elseif [string match "$mom_sim_nc_func(RETURN_HOME)" $code] {

           # Coord specified with this command is stored as the "intermediate point" that
           # the tool will position in rapid mode before going to the reference point.
           # Intermediate point (each axis) stays modal for future G28/G29.
           #
           # This command cancels cutcom and tool offsets.
           #
           # Axes of the "intermediate point" only exist when specified. Default it to
           # the ref. pt. will not be sufficient.
           # The int. pt. floats with respect to the active WCS (G54-59).
           # Reference point is fixed on a machine.
   
            set mom_sim_nc_register(RETURN_HOME) 1

         } elseif [string match "$mom_sim_nc_func(RETURN_HOME_P)" $code] {

           # G30 P2 XYZ -- Return to the 2nd ref pt. P2 can be omitted.
           # G30 P3 XYZ -- Return to the 3rd ref pt.
           # G30 P4 XYZ -- Return to the 4th ref pt.
           #
           # G30 & G28 share the same intermediate point. I think?.

         } elseif [string match "$mom_sim_nc_func(FROM_HOME)" $code] {

           # This command is normally issued immediately after a G28.
           #
           # Coord specified with this function is the target that the tool will position.
           # The tool will first position from the ref. pt. to the "intermediate point"
           # that have been defined in the previous G28 command, if any, in rapid mode
           # before going to its final destination.

            set mom_sim_nc_register(FROM_HOME) 1

         } elseif [string match "$mom_sim_nc_func(MACH_CS_MOVE)" $code] {

           # Moves to a position in the Machine Coordiante System (MCS).
           # The origin of MCS (Machine Zero, Machine Datum) never changes.
           # One shot deal block. Ignored in incremental mode (G91).
           # MCS is never affected by G92 or G52 (LCS) until machine is powered off.
           # (I think, guess), G53 is used in the M06 macro for the tool change.

            set mom_sim_nc_register(MCS_MOVE) 1

           # Fake WCS offset. Restore it after move.
            set mom_sim_nc_register(WORK_OFFSET) [list]
            lappend mom_sim_nc_register(WORK_OFFSET) 0.0 0.0 0.0 0.0 0.0

         } elseif [string match "$mom_sim_nc_func(WORK_CS_1)" $code] {

           # Select Work Coordinate System.
           # Each set of offsets (from Machine Zero) is entered and stored in the machine.
           # G54 is selected when powered on (?).
           # Work zero point offset value can be set in a program by G10 command (??).

            set mom_sim_nc_register(WCS) 1
            set mom_sim_nc_register(WORK_OFFSET) [list]
            foreach coord $mom_sim_wcs_offsets($mom_sim_nc_register(WCS)) {
               lappend mom_sim_nc_register(WORK_OFFSET) $coord
            }

         } elseif [string match "$mom_sim_nc_func(WORK_CS_2)" $code] {

            set mom_sim_nc_register(WCS) 2
            set mom_sim_nc_register(WORK_OFFSET) [list]
            foreach coord $mom_sim_wcs_offsets($mom_sim_nc_register(WCS)) {
               lappend mom_sim_nc_register(WORK_OFFSET) $coord
            }

         } elseif [string match "$mom_sim_nc_func(WORK_CS_3)" $code] {

            set mom_sim_nc_register(WCS) 3
            set mom_sim_nc_register(WORK_OFFSET) [list]
            foreach coord $mom_sim_wcs_offsets($mom_sim_nc_register(WCS)) {
               lappend mom_sim_nc_register(WORK_OFFSET) $coord
            }
   
         } elseif [string match "$mom_sim_nc_func(WORK_CS_4)" $code] {

            set mom_sim_nc_register(WCS) 4
            set mom_sim_nc_register(WORK_OFFSET) [list]
            foreach coord $mom_sim_wcs_offsets($mom_sim_nc_register(WCS)) {
               lappend mom_sim_nc_register(WORK_OFFSET) $coord
            }

         } elseif [string match "$mom_sim_nc_func(WORK_CS_5)" $code] {

            set mom_sim_nc_register(WCS) 5
            set mom_sim_nc_register(WORK_OFFSET) [list]
            foreach coord $mom_sim_wcs_offsets($mom_sim_nc_register(WCS)) {
               lappend mom_sim_nc_register(WORK_OFFSET) $coord
            }

         } elseif [string match "$mom_sim_nc_func(WORK_CS_6)" $code] {

            set mom_sim_nc_register(WCS) 6
            set mom_sim_nc_register(WORK_OFFSET) [list]
            foreach coord $mom_sim_wcs_offsets($mom_sim_nc_register(WCS)) {
               lappend mom_sim_nc_register(WORK_OFFSET) $coord
            }

         } elseif [string match "$mom_sim_nc_func(LOCAL_CS_SET)" $code] {

           # Shift all WCS. Offsets are specified.
           #
           # Specify a local coordiante system within the active work coordinate system.
           # Shifts will be applied to All WCS.
           # G52 IP0 or G92 command cancels this command.

         set mom_sim_nc_register(SET_LOCAL) 1

         } elseif [string match "$mom_sim_nc_func(WORK_CS_RESET)" $code] {

           # Disable work coord reset if S is in the block.
           # G92 is used to set max spindle speed.
            global mom_sim_address
            if [VNC_parse_nc_block mom_sim_o_buffer $mom_sim_address(S,leader)] {
return
            }


           # Modify Work Coordinate System (G54 - G59).
           #
           # This command immediately make current pos a new origin of the working coordiante system
           # The offsets are added to ALL work coordinate systems (G54 - G59).
           # Subsequent reference to any of the WCS will have the offsets in effect.
           #
           # XYZ specified with this command will be used to position the tool from new origin.
           # A Manual Reference Point Return cancels G92. All WCS return to normal (??).

           # Defer offsets calculation until entire block is parsed in VNC_sim_nc_block.
            set mom_sim_nc_register(RESET_WCS) 1


global mom_sim_message
set mom_sim_message "-- RESET WCS"
PB_CMD_vnc__send_message

         }

         break
      }
   }
}


#=============================================================
proc PB_CMD_vnc__G_mode_code { } {
#=============================================================
  global mom_sim_nc_register
  global mom_sim_nc_func


   set codes_list [list]
    lappend codes_list $mom_sim_nc_func(INPUT_ABS)
    lappend codes_list $mom_sim_nc_func(INPUT_INC)


   global mom_sim_o_buffer
   foreach code $codes_list {

      if { [VNC_parse_nc_block mom_sim_o_buffer $code] == 1 } {

         if [string match "$mom_sim_nc_func(INPUT_ABS)" $code] {

            set mom_sim_nc_register(INPUT) ABS

         } elseif [string match "$mom_sim_nc_func(INPUT_INC)" $code] {

            set mom_sim_nc_register(INPUT) INC
         }

         break
      }
   }
}


#=============================================================
proc PB_CMD_vnc__G_motion_code { } {
#=============================================================
  global mom_sim_nc_register
  global mom_sim_nc_func


   set codes_list [list]
    lappend codes_list $mom_sim_nc_func(MOTION_RAPID)
    lappend codes_list $mom_sim_nc_func(MOTION_LINEAR)
    lappend codes_list $mom_sim_nc_func(MOTION_CIRCULAR_CLW)
    lappend codes_list $mom_sim_nc_func(MOTION_CIRCULAR_CCLW)
    lappend codes_list $mom_sim_nc_func(CYCLE_DRILL)
    lappend codes_list $mom_sim_nc_func(CYCLE_DRILL_DWELL)
    lappend codes_list $mom_sim_nc_func(CYCLE_DRILL_DEEP)
    lappend codes_list $mom_sim_nc_func(CYCLE_DRILL_BREAK_CHIP)
    lappend codes_list $mom_sim_nc_func(CYCLE_TAP)
    lappend codes_list $mom_sim_nc_func(CYCLE_BORE)
    lappend codes_list $mom_sim_nc_func(CYCLE_BORE_DRAG)
    lappend codes_list $mom_sim_nc_func(CYCLE_BORE_NO_DRAG)
    lappend codes_list $mom_sim_nc_func(CYCLE_BORE_BACK)
    lappend codes_list $mom_sim_nc_func(CYCLE_BORE_MANUAL)
    lappend codes_list $mom_sim_nc_func(CYCLE_BORE_MANUAL_DWELL)
    lappend codes_list $mom_sim_nc_func(CYCLE_START)
    lappend codes_list $mom_sim_nc_func(CYCLE_OFF)


   global mom_sim_o_buffer
   foreach code $codes_list {

      if { [VNC_parse_nc_block mom_sim_o_buffer $code] == 1 } {

         if [string match "$mom_sim_nc_func(MOTION_RAPID)"               $code] {

            set mom_sim_nc_register(MOTION) "RAPID"

         } elseif [string match "$mom_sim_nc_func(MOTION_LINEAR)"        $code] {

            set mom_sim_nc_register(MOTION) "LINEAR"

         } elseif [string match "$mom_sim_nc_func(MOTION_CIRCULAR_CLW)"  $code] {

            set mom_sim_nc_register(MOTION) "CIRCULAR_CLW"

         } elseif [string match "$mom_sim_nc_func(MOTION_CIRCULAR_CCLW)" $code] {

            set mom_sim_nc_register(MOTION) "CIRCULAR_CCLW"

         } elseif [string match "$mom_sim_nc_func(CYCLE_DRILL)"          $code] {

            set mom_sim_nc_register(MOTION) "CYCLE_DRILL"

         } elseif [string match "$mom_sim_nc_func(CYCLE_DRILL_DWELL)"    $code] {

            set mom_sim_nc_register(MOTION) "CYCLE_DRILL_DWELL"

         } elseif [string match "$mom_sim_nc_func(CYCLE_DRILL_DEEP)"     $code] {

            set mom_sim_nc_register(MOTION) "CYCLE_DRILL_DEEP"

         } elseif [string match "$mom_sim_nc_func(CYCLE_DRILL_BREAK_CHIP)"  $code] {

            set mom_sim_nc_register(MOTION) "CYCLE_DRILL_BREAK_CHIP"

         } elseif [string match "$mom_sim_nc_func(CYCLE_TAP)"               $code] {

            set mom_sim_nc_register(MOTION) "CYCLE_TAP"

         } elseif [string match "$mom_sim_nc_func(CYCLE_BORE)"              $code] {

            set mom_sim_nc_register(MOTION) "CYCLE_BORE"

         } elseif [string match "$mom_sim_nc_func(CYCLE_BORE_DRAG)"         $code] {

            set mom_sim_nc_register(MOTION) "CYCLE_BORE_DRAG"

         } elseif [string match "$mom_sim_nc_func(CYCLE_BORE_NO_DRAG)"      $code] {

            set mom_sim_nc_register(MOTION) "CYCLE_BORE_NO_DRAG"

         } elseif [string match "$mom_sim_nc_func(CYCLE_BORE_BACK)"         $code] {

            set mom_sim_nc_register(MOTION) "CYCLE_BORE_BACK"

         } elseif [string match "$mom_sim_nc_func(CYCLE_BORE_MANUAL)"       $code] {

            set mom_sim_nc_register(MOTION) "CYCLE_BORE_MANUAL"

         } elseif [string match "$mom_sim_nc_func(CYCLE_BORE_MANUAL_DWELL)" $code] {

            set mom_sim_nc_register(MOTION) "CYCLE_BORE_MANUAL_DWELL"

         } elseif [string match "$mom_sim_nc_func(CYCLE_START)"             $code] {

            set mom_sim_nc_register(MOTION) "CYCLE_START"

         } elseif [string match "$mom_sim_nc_func(CYCLE_OFF)"               $code] {

            set mom_sim_nc_register(MOTION) "CYCLE_OFF"

         }

         break
      }
   }
}


#=============================================================
proc PB_CMD_vnc__G_plane_code { } {
#=============================================================
  global mom_sim_nc_register
  global mom_sim_nc_func
  global mom_sim_cycle_spindle_axis


   set codes_list [list]
    lappend codes_list $mom_sim_nc_func(PLANE_XY)
    lappend codes_list $mom_sim_nc_func(PLANE_YZ)
    lappend codes_list $mom_sim_nc_func(PLANE_ZX)


   global mom_sim_o_buffer
   foreach code $codes_list {

      if { [VNC_parse_nc_block mom_sim_o_buffer $code] == 1 } {

         if [string match "$mom_sim_nc_func(PLANE_XY)"        $code] {

            set mom_sim_nc_register(PLANE) XY
            set mom_sim_cycle_spindle_axis 2

         } elseif [string match "$mom_sim_nc_func(PLANE_YZ)"  $code] {

            set mom_sim_nc_register(PLANE) YZ
            set mom_sim_cycle_spindle_axis 0

         } elseif [string match "$mom_sim_nc_func(PLANE_ZX)"  $code] {

            set mom_sim_nc_register(PLANE) ZX
            set mom_sim_cycle_spindle_axis 1
         }

         break
      }
   }
}


#=============================================================
proc PB_CMD_vnc__G_spin_code { } {
#=============================================================
  global mom_sim_nc_register
  global mom_sim_nc_func


   set codes_list [list]
    lappend codes_list $mom_sim_nc_func(SPEED_SFM)
    lappend codes_list $mom_sim_nc_func(SPEED_SMM)
    lappend codes_list $mom_sim_nc_func(SPEED_RPM)


   global mom_sim_o_buffer
   foreach code $codes_list {

      if { [VNC_parse_nc_block mom_sim_o_buffer $code] == 1 } {

         if [string match "$mom_sim_nc_func(SPEED_SFM)" $code] {

            set mom_sim_nc_register(SPINDLE_MODE) SFM

         } elseif [string match "$mom_sim_nc_func(SPEED_SMM)"  $code] {

            set mom_sim_nc_register(SPINDLE_MODE) SMM

         } elseif [string match "$mom_sim_nc_func(SPEED_RPM)"  $code] {

            set mom_sim_nc_register(SPINDLE_MODE) REV_PER_MIN
         }

         break
      }
   }
}


#=============================================================
proc PB_CMD_vnc__M_misc_code { } {
#=============================================================
  global mom_sim_nc_register
  global mom_sim_nc_func


   set codes_list [list]
    lappend codes_list $mom_sim_nc_func(PROG_STOP)
    lappend codes_list $mom_sim_nc_func(PROG_OPSTOP)
    lappend codes_list $mom_sim_nc_func(PROG_END)
    lappend codes_list $mom_sim_nc_func(TOOL_CHANGE)
    lappend codes_list $mom_sim_nc_func(PROG_STOP_REWIND)


   global mom_sim_o_buffer

   foreach code $codes_list {

      if { [VNC_parse_nc_block mom_sim_o_buffer $code] == 1 } {

         if [string match "$mom_sim_nc_func(PROG_STOP)"         $code] {

         } elseif [string match "$mom_sim_nc_func(PROG_OPSTOP)" $code] {

         } elseif [string match "$mom_sim_nc_func(PROG_END)"    $code] {

           # Reset following functions on NC controller to the initial state.
           # (Fanuc System 9 Series)
           #
           #  G22 - Store stroke limit function ON
           #  G40 - Cutcom OFF
           #  G49 - Tool length adjust OFF
           #  G50 - Scaling OFF
           #  G54 - Work coordinate system 1
           #  G64 - Cutting mode
           #  G67 - Macro modal call cancel
           #  G69 - Coordinate system rotation OFF
           #  G80 - Canned cycle cancel
           #  G98 - Return to initial level (cycle)

            VNC_reset_controller

         } elseif [string match "$mom_sim_nc_func(TOOL_CHANGE)"      $code] {

            VNC_tool_change

         } elseif [string match "$mom_sim_nc_func(PROG_STOP_REWIND)" $code] {

            VNC_rewind_stop_program

         } else {
         }

         break
      }
   }
}


#=============================================================
proc PB_CMD_vnc__M_spindle_code { } {
#=============================================================
  global mom_sim_nc_register
  global mom_sim_nc_func


   set codes_list [list]
    lappend codes_list $mom_sim_nc_func(SPINDLE_CLW)
    lappend codes_list $mom_sim_nc_func(SPINDLE_CCLW)
    lappend codes_list $mom_sim_nc_func(SPINDLE_OFF)


   global mom_sim_o_buffer
   foreach code $codes_list {

      if { [VNC_parse_nc_block mom_sim_o_buffer $code] == 1 } {

         if [string match "$mom_sim_nc_func(SPINDLE_CLW)" $code] {

            set mom_sim_nc_register(SPINDLE_DIRECTION) CLW

         } elseif [string match "$mom_sim_nc_func(SPINDLE_CCLW)"  $code] {

            set mom_sim_nc_register(SPINDLE_DIRECTION) CCLW

         } elseif [string match "$mom_sim_nc_func(SPINDLE_OFF)"  $code] {

            set mom_sim_nc_register(SPINDLE_DIRECTION) OFF
         }

         break
      }
   }
}


#=============================================================
proc PB_CMD_vnc__R_code { } {
#=============================================================
  global mom_sim_nc_register

   set mom_sim_nc_register(R_IN_USE) 1
}


#=============================================================
proc PB_CMD_vnc__T_code { } {
#=============================================================
  global mom_sim_pos mom_sim_prev_pos

   PB_CMD_vnc__tool_change
}


#=============================================================
proc PB_CMD_vnc____map_machine_tool_axes { } {
#=============================================================
  global mom_sim_address
  global mom_sim_num_machine_axes
  global mom_sim_mt_axis
  global mom_sim_result mom_sim_result1
  global mom_sim_zcs_base
  global mom_sim_spindle_comp mom_sim_spindle_jct
  global mom_sim_reverse_4th_table mom_sim_reverse_5th_table
  global mom_sim_machine_head



#####################################################
#
# Configure parameters set between ##>>>>> & ##<<<<<.
#
#####################################################

##>>>>>
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # When desired, override the default number of machine axes
  # to reduce the number of addressable axes.
  # This parameter has been set according to the post already.
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  ## set mom_sim_num_machine_axes 5

  #++++++++++++++++++++++++++++++++++++++++++
  # Fetch physical axes (machine tool model). 
  #++++++++++++++++++++++++++++++++++++++++++
   set mom_sim_mt_axis(X) X
   set mom_sim_mt_axis(Y) Y
   set mom_sim_mt_axis(Z) Z

   switch $mom_sim_num_machine_axes {
      "4" {
         set mom_sim_mt_axis(4) A
      }
      "5" {
         set mom_sim_mt_axis(4) B
         set mom_sim_mt_axis(5) C1
      }
   }

  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Specify a component as ZCS base to be referenced in PB_VNC_csys.
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   set mom_sim_zcs_base "MACHINE_BASE"

  #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Define spindle component and its junction for mounting tools.
  #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   set mom_sim_spindle_comp "SPINDLE"
   set mom_sim_spindle_jct  "MILL_TOOL_MOUNT_JCT"

  #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Reverse axis of rotation for 4th and/or 5th rotary table,
  # if they are not set properly in the machine tool model.
  #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   if { $mom_sim_num_machine_axes > 3 } {
      set mom_sim_reverse_4th_table  0
      set mom_sim_reverse_5th_table  1
   }

  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # If necessary, define junction for advanced kinematics
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++
  global mom_sim_advanced_kinematic_jct
  ##   set mom_sim_advanced_kinematic_jct "ROT_JCT"
##<<<<<



  # For this machine that all physical axes participate in executing a motion. 
  # Include all logical names for axes.

  # Map logical names (addresses) to physical axes (machine tool model). 
   SIM_set_linear_axes_map [concat $mom_sim_address(X,leader) $mom_sim_mt_axis(X)  $mom_sim_address(Y,leader) $mom_sim_mt_axis(Y)  $mom_sim_address(Z,leader) $mom_sim_mt_axis(Z)]

   SIM_set_linear_axes_config  [concat $mom_sim_mt_axis(X) $mom_sim_mt_axis(Y) $mom_sim_mt_axis(Z)]

   switch $mom_sim_num_machine_axes {
      "4" {
         SIM_set_rotary_axes_map     [concat $mom_sim_address(fourth_axis,leader) $mom_sim_mt_axis(4)]
         SIM_set_rotary_axes_config  [concat $mom_sim_mt_axis(4)]
      }
      "5" {
         SIM_set_rotary_axes_map     [concat $mom_sim_address(fourth_axis,leader) $mom_sim_mt_axis(4)  $mom_sim_address(fifth_axis,leader)  $mom_sim_mt_axis(5)]
         SIM_set_rotary_axes_config  [concat $mom_sim_mt_axis(4) $mom_sim_mt_axis(5)]
      }
   }
}


#=============================================================
proc PB_CMD_vnc____set_nc_definitions { } {
#=============================================================
#
# User adds his own settings here.
#
  global mom_sim_word_separator mom_sim_word_separator_len
  global mom_sim_end_of_block mom_sim_end_of_block_len
  global mom_sim_address
  global mom_sim_wcs_offsets
  global mom_sim_control_out mom_sim_control_in
  global mom_sim_nc_register
  global mom_sim_nc_func
  global mom_sim_prog_rewind_stop_code
  global mom_sim_other_nc_codes
  global mom_sim_incr_linear_addrs
  global mom_sim_control_var_leader


################################################
#
# Add other NC codes that need to be simulated
# but may have not been defined in Post Builder.
#
################################################

  #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Define program rewind stop code here.  This will not be needed
  # in the future when UI for this code is implemented in PB.
  #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   set mom_sim_prog_rewind_stop_code "%"

  #++++++++++++++++++++++++++++++++++++++++++++
  # Define leader for the controller variables.
  #++++++++++++++++++++++++++++++++++++++++++++
   set mom_sim_control_var_leader    "#"

  #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Set default N/C input mode.  This value will be used
  # when no input mode function is specified in the N/C code.
  # It's also used as the intial mode when controller is reset.
  #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   set mom_sim_nc_register(INPUT)          ABS
   set mom_sim_nc_register(DEF_INPUT)      $mom_sim_nc_register(INPUT)

  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Specify NC codes that will cause movement of other devices.
  # Reaction to each code should be defined in  PB_CMD_vnc____sim_other_devides.
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   set mom_sim_other_nc_codes {T G110 G111 M201 M202}

  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Specify MOM Addresses used to command incremental linear axis movements.
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   set mom_sim_incr_linear_addrs {U V W}

  #+++++++++++++++++++++++++++++++++++++++++++
  # Other G codes not defined in Post Builder
  # but referenced in simulation.
  #+++++++++++++++++++++++++++++++++++++++++++
   set mom_sim_nc_func(RETURN_HOME_P)       "30"
   set mom_sim_nc_func(FROM_HOME)           "29"
   set mom_sim_nc_func(MACH_CS_MOVE)        "53"
   set mom_sim_nc_func(WORK_CS_1)           "54"
   set mom_sim_nc_func(WORK_CS_2)           "55"
   set mom_sim_nc_func(WORK_CS_3)           "56"
   set mom_sim_nc_func(WORK_CS_4)           "57"
   set mom_sim_nc_func(WORK_CS_5)           "58"
   set mom_sim_nc_func(WORK_CS_6)           "59"
   set mom_sim_nc_func(LOCAL_CS_SET)        "52"

  #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Actual Work coordinate offsets may need to be defined here
  # if they can not be passed from a UG/CAM Program.
  #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   set mom_sim_wcs_offsets(0) [list]
   set mom_sim_wcs_offsets(1) [list]
   set mom_sim_wcs_offsets(2) [list]
   set mom_sim_wcs_offsets(3) [list]
   set mom_sim_wcs_offsets(4) [list]
   set mom_sim_wcs_offsets(5) [list]
   set mom_sim_wcs_offsets(6) [list]
   lappend mom_sim_wcs_offsets(0) 0.0 0.0 0.0 0.0 0.0
   lappend mom_sim_wcs_offsets(1) 0.0 0.0 0.0 0.0 0.0
   lappend mom_sim_wcs_offsets(2) 0.0 0.0 0.0 0.0 0.0
   lappend mom_sim_wcs_offsets(3) 0.0 0.0 0.0 0.0 0.0
   lappend mom_sim_wcs_offsets(4) 0.0 0.0 0.0 0.0 0.0
   lappend mom_sim_wcs_offsets(5) 0.0 0.0 0.0 0.0 0.0
   lappend mom_sim_wcs_offsets(6) 0.0 0.0 0.0 0.0 0.0
}


#=============================================================
proc PB_CMD_vnc____sim_other_devices { } {
#=============================================================
  global mom_sim_o_buffer
  global mom_sim_result mom_sim_result1
  global mom_sim_tool_loaded
  global mom_sim_spindle_comp mom_sim_spindle_jct
  global mom_sim_nc_register
  global mom_sim_address mom_sim_format

  global mom_sim_other_nc_codes
  global mom_sim_nc_address
  global mom_sim_nc_code
  global mom_sim_pos mom_sim_prev_pos
  global mom_sim_simulate_block


  #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Specify action for each code encountered.
  #
  # Codes to be simulated are defined by mom_sim_other_nc_codes in
  # PB_CMD_vnc____set_nc_definitions.
  #
  # Example here has been defined for Mazak Integrex 200.
  #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

   switch $mom_sim_nc_address {

      T    { ;# Tool change
         PB_CMD_vnc__tool_change

        # Restore 4th axis (B) register, if it's present in the block for tool change purpose.
         if {[VNC_parse_nc_code $mom_sim_o_buffer $mom_sim_address(fourth_axis,leader)] == 1 } {
            set mom_sim_pos(3) $mom_sim_prev_pos(3)
         }

        # Disable further simulation
         set mom_sim_simulate_block 0
      }

      G110 { ;# Cross machining starts

         set mom_sim_nc_register(CROSS_MACHINING) 1
         VNC_restore_pos_no_sim

        # Sub spindle unclamp
         set jaw1 CHUCK1_JAW1
         set jaw2 CHUCK1_JAW2
         set jaw3 CHUCK1_JAW3

         SIM_ask_axis_position $jaw1
         set jaw1_pos $mom_sim_result
         SIM_ask_axis_position $jaw2
         set jaw2_pos $mom_sim_result
         SIM_ask_axis_position $jaw3
         set jaw3_pos $mom_sim_result

         set jaw1 CHUCK2_JAW1
         set jaw2 CHUCK2_JAW2
         set jaw3 CHUCK2_JAW3

         set relay1 100
         set relay2 800

         SIM_move_linear_axis $relay1 $jaw1 1.5
         SIM_move_linear_axis $relay1 $jaw2 1.5
         SIM_move_linear_axis $relay1 $jaw3 1.5
         SIM_update

        # Move subspindle to part
         SIM_mount_kim_comp $relay2 Z_SUB_RAM CHUCK2 PART PART_SUB_JCT
         SIM_update

        # Sub spindle clamp
         SIM_move_linear_axis $relay1 $jaw1 $jaw1_pos
         SIM_move_linear_axis $relay1 $jaw2 $jaw2_pos
         SIM_move_linear_axis $relay1 $jaw3 $jaw3_pos
         SIM_update

        # Main spindle unclamp
         set jaw1 CHUCK1_JAW1
         set jaw2 CHUCK1_JAW2
         set jaw3 CHUCK1_JAW3

         SIM_move_linear_axis $relay1 $jaw1 1.5
         SIM_move_linear_axis $relay1 $jaw2 1.5
         SIM_move_linear_axis $relay1 $jaw3 1.5
         SIM_update

        # Return subspindle
         SIM_mount_kim_comp $relay2 Z_SUB_RAM Z_SUB_RAM MACHINE_BASE Z_SUB_RAM_BASE
         SIM_mount_kim_comp $relay2 PART PART_SUB_JCT Z_SUB_RAM CHUCK2
         SIM_update
      }

      G111 { ;# Cross machining ends

         set mom_sim_nc_register(CROSS_MACHINING) 0

        # Zero Z register to prepare Local WCS offset for subspindle
         set mom_sim_pos(2) 0
      }

      M200 -
      M201 { ;# Milling mode

         set mom_sim_nc_register(MACHINE_MODE) MILL
      }

      M202 { ;# Turning mode
         set mom_sim_nc_register(MACHINE_MODE) TURN

        # Zero Y.
         global mom_sim_pos
         set mom_sim_pos(1) 0
      }
    }
}


#=============================================================
proc PB_CMD_vnc__ask_feedrate_mode { } {
#=============================================================
  global mom_feed_rate_mode

   if [info exists mom_feed_rate_mode] {
      switch $mom_feed_rate_mode {
         IPM     { set feed_units "INCH_PER_MIN" }
         MMPM    { set feed_units "MM_PER_MIN"}
         IPR     { set feed_units "INCH_PER_REV" }
         MMPR    { set feed_units "MM_PER_REV" }
         FRN     { set feed_units "MM_PER_100REV" }
         default { set feed_units "UNKNOWN FEEDRATE UNITS" }
      }
   } else {
      set feed_units "UNKNOWN FEEDRATE UNITS" 
   }

return $feed_units
}


#=============================================================
proc PB_CMD_vnc__ask_speed_mode { } {
#=============================================================
  global mom_spindle_mode

   if [info exists mom_spindle_mode] {
      switch $mom_spindle_mode {
         RPM     { set speed_units "REV_PER_MIN" }
#        SFM     {}
#        SMM     {}
         default { set speed_units "UNKNOWN SPEED UNITS" }
      }
   } else {
      set speed_units "UNKNOWN SPEED UNITS" 
   }

return $speed_units
}


#=============================================================
proc PB_CMD_vnc__calculate_duration_time { } {
#=============================================================
  global mom_sim_motion_linear_or_angular mom_sim_travel_delta mom_sim_duration_time

  global mom_feed_rate
  global sim_feedrate_mode
  global mom_sim_nc_register
  global mom_sim_rapid_feed_rate


   set linear_or_angular $mom_sim_motion_linear_or_angular
   set delta $mom_sim_travel_delta

  # For now, we don't compute durations for angular motions
   if { $linear_or_angular == "ANGULAR" } {
      set mom_sim_duration_time 0
return
   }

   if { $sim_feedrate_mode == "RAPID" } {
      set feed $mom_sim_rapid_feed_rate
   } else {
      set feed $mom_feed_rate
   }

   set length [expr abs($delta)]

   if {[expr abs($feed)] < 0.00001 || $length < 0.00001 } {
      set time 0
   } else {
      set time [expr ($length / $feed) * 60.0]
   }

   set mom_sim_duration_time $time
}


#=============================================================
proc PB_CMD_vnc__circular_move { } {
#=============================================================
  global mom_sim_address mom_sim_pos mom_sim_prev_pos
  global mom_sim_lg_axis
  global mom_sim_nc_register
  global mom_sim_circular_dir
  global mom_sim_circular_vector

  global mom_sim_o_buffer


  # Search for R word
   if [info exists mom_sim_nc_register(R_IN_USE)] {

     # Unset it only if not modal!
      if { !$mom_sim_address(R,modal) } {
         unset mom_sim_nc_register(R_IN_USE)
      }
   }

   VNC_parse_nc_block mom_sim_o_buffer $mom_sim_address(R,leader) PB_CMD_vnc__R_code


   VNC_set_feedrate_mode "CUT"

   set dir $mom_sim_circular_dir
   set plane $mom_sim_nc_register(PLANE)


  # When "Unsigned Vector - Arc Start to Center" is used to define
  # the arc center vector, fake it into R case for arc center calculation.

   if [string match "Unsigned Vector - Arc Start to Center" $mom_sim_circular_vector] {

      set mom_sim_nc_register(R_IN_USE) 1

      switch $plane {
         "YZ" {
            set dx $mom_sim_pos(6)
            set dy $mom_sim_pos(7)
         }
         "ZX" {
            set dx $mom_sim_pos(7)
            set dy $mom_sim_pos(5)
         }
         default {
            set dx $mom_sim_pos(5)
            set dy $mom_sim_pos(6)
         }
      }

     # Arc radius
      set mom_sim_nc_register(R) [expr sqrt($dx*$dx + $dy*$dy)]
   }


  # Find arc center vector
   if [info exists mom_sim_nc_register(R_IN_USE)] {

      switch $plane {
         "YZ" {
            set dx [expr $mom_sim_pos(1) - $mom_sim_prev_pos(1)]
            set dy [expr $mom_sim_pos(2) - $mom_sim_prev_pos(2)]
         }
         "ZX" {
            set dx [expr $mom_sim_pos(2) - $mom_sim_prev_pos(2)]
            set dy [expr $mom_sim_pos(0) - $mom_sim_prev_pos(0)]
         }
         default {
            set dx [expr $mom_sim_pos(0) - $mom_sim_prev_pos(0)]
            set dy [expr $mom_sim_pos(1) - $mom_sim_prev_pos(1)]
         }
      }

      set r $mom_sim_nc_register(R)
      set a [expr atan2($dy,$dx)]
      set b [expr acos(sqrt($dx*$dx + $dy*$dy) / (2.0*$r))]

     # Complement of an arc
      if [expr $r < 0] {
         set dir [expr -1 * $dir]
      }

      set c [expr $a + $dir*$b]

      switch $plane {
         "YZ" {
            set j [expr $r * cos($c) + $mom_sim_prev_pos(1)]
            set k [expr $r * sin($c) + $mom_sim_prev_pos(2)]
         }
         "ZX" {
            set k [expr $r * cos($c) + $mom_sim_prev_pos(2)]
            set i [expr $r * sin($c) + $mom_sim_prev_pos(0)]
         }
         default {
            set i [expr $r * cos($c) + $mom_sim_prev_pos(0)]
            set j [expr $r * sin($c) + $mom_sim_prev_pos(1)]
         }
      }

   } else {

      switch $mom_sim_circular_vector {
         "Vector - Arc Start to Center" {
            set i [expr $mom_sim_prev_pos(0) + $mom_sim_pos(5)]
            set j [expr $mom_sim_prev_pos(1) + $mom_sim_pos(6)]
            set k [expr $mom_sim_prev_pos(2) + $mom_sim_pos(7)]
         }

         "Vector - Arc Center to Start" {
            set i [expr $mom_sim_prev_pos(0) - $mom_sim_pos(5)]
            set j [expr $mom_sim_prev_pos(1) - $mom_sim_pos(6)]
            set k [expr $mom_sim_prev_pos(2) - $mom_sim_pos(7)]
         }

         "Vector - Absolute Arc Center" {
            set i $mom_sim_pos(5)
            set j $mom_sim_pos(6)
            set k $mom_sim_pos(7)
         }
      }
   }

   global mom_sim_arc_output_mode
   global mom_sim_PI

   set helix_pitch ""

   switch $plane {
      "YZ" {

        # Handle helical condition
         if [expr $mom_sim_pos(0) != $mom_sim_prev_pos(0)] {
            if { [info exists mom_sim_helix_pitch] && [expr $mom_sim_helix_pitch != 0.0] } {
               set helix_pitch "P $mom_sim_helix_pitch"
            }
         }

        # Handle full circle case using half circle pt for non-helix (???)
         if [string match "" $helix_pitch] {

            if [string match "FULL_CIRCLE" $mom_sim_arc_output_mode] {
               if { [expr $mom_sim_pos(1) == $mom_sim_prev_pos(1)] && 
                    [expr $mom_sim_pos(2) == $mom_sim_prev_pos(2)] } {

                  set dx [expr $mom_sim_pos(1) - $j]
                  set dy [expr $mom_sim_pos(2) - $k]
                  set a [expr atan2($dy,$dx)]
                  set b [expr $a + $mom_sim_PI]
                  set r [expr sqrt($dx*$dx + $dy*$dy)]
                  set px [expr $j + $r*cos($b)]
                  set py [expr $k + $r*sin($b)]

                  SIM_move_circular_zcs $dir 0 0 $mom_sim_lg_axis(Y) $px $mom_sim_lg_axis(Z) $py $mom_sim_address(J,leader) $j $mom_sim_address(K,leader) $k
               }
            }
         }

         eval SIM_move_circular_zcs $dir 0 0 $mom_sim_lg_axis(Y) $mom_sim_pos(1) $mom_sim_lg_axis(Z) $mom_sim_pos(2) $mom_sim_address(J,leader) $j $mom_sim_address(K,leader) $k $helix_pitch
      }

      "ZX" {

        # Handle helical condition
         if [expr $mom_sim_pos(1) != $mom_sim_prev_pos(1)] {
            if { [info exists mom_sim_helix_pitch] && [expr $mom_sim_helix_pitch != 0.0] } {
               set helix_pitch "P $mom_sim_helix_pitch"
            }
         }

        # Handle full circle case using half circle pt for non-helix
         if [string match "" $helix_pitch] {
            if [string match "FULL_CIRCLE" $mom_sim_arc_output_mode] {
               if { [expr $mom_sim_pos(2) == $mom_sim_prev_pos(2)] && 
                    [expr $mom_sim_pos(0) == $mom_sim_prev_pos(0)] } {

                  set dx [expr $mom_sim_pos(2) - $k]
                  set dy [expr $mom_sim_pos(0) - $i]
                  set a [expr atan2($dy,$dx)]
                  set b [expr $a + $mom_sim_PI]
                  set r [expr sqrt($dx*$dx + $dy*$dy)]
                  set px [expr $k + $r*cos($b)]
                  set py [expr $i + $r*sin($b)]

                  SIM_move_circular_zcs 0 $dir 0 $mom_sim_lg_axis(Z) $px $mom_sim_lg_axis(X) $py $mom_sim_address(K,leader) $k $mom_sim_address(I,leader) $i
               }
            }
         }
         eval SIM_move_circular_zcs 0 $dir 0 $mom_sim_lg_axis(Z) $mom_sim_pos(2) $mom_sim_lg_axis(X) $mom_sim_pos(0) $mom_sim_address(K,leader) $k $mom_sim_address(I,leader) $i $helix_pitch
      }

      default {

        # Handle helical condition
         if [expr $mom_sim_pos(2) != $mom_sim_prev_pos(2)] {
            if { [info exists mom_sim_helix_pitch] && [expr $mom_sim_helix_pitch != 0.0] } {
               set helix_pitch "P $mom_sim_helix_pitch"
            }
         }

        # Handle full circle case using half circle pt for non-helix
         if [string match "" $helix_pitch] {
            if [string match "FULL_CIRCLE" $mom_sim_arc_output_mode] {
               if { [expr $mom_sim_pos(0) == $mom_sim_prev_pos(0)] && 
                    [expr $mom_sim_pos(1) == $mom_sim_prev_pos(1)] } {

                  set dx [expr $mom_sim_pos(0) - $i]
                  set dy [expr $mom_sim_pos(1) - $j]
                  set a [expr atan2($dy,$dx)]
                  set b [expr $a + $mom_sim_PI]
                  set r [expr sqrt($dx*$dx + $dy*$dy)]
                  set px [expr $i + $r*cos($b)]
                  set py [expr $j + $r*sin($b)]

                  SIM_move_circular_zcs 0 0 $dir $mom_sim_lg_axis(X) $px $mom_sim_lg_axis(Y) $py $mom_sim_address(I,leader) $i $mom_sim_address(J,leader) $j
               }
            }
         }

         eval  SIM_move_circular_zcs 0 0 $dir $mom_sim_lg_axis(X) $mom_sim_pos(0) $mom_sim_lg_axis(Y) $mom_sim_pos(1) $mom_sim_address(I,leader) $i $mom_sim_address(J,leader) $j $helix_pitch
      }
   }
}


#=============================================================
proc PB_CMD_vnc__config_sync_dialog { } {
#=============================================================
  global mom_sim_vnc_msg_only

   if { ![info exists mom_sim_vnc_msg_only] || !$mom_sim_vnc_msg_only } {

      SIM_dialog_start		"SYS_MAIN_CONTROL_PANEL TITLE=<Merging-Lathe Control Panel>"

	SIM_dialog_add_item	"SYS_MACHINE_TIME_ITEM		S1"
	SIM_dialog_add_item	"SYS_CHANNEL_ITEM		S2 ORIENTATION=<HORIZONTAL> SPAN=<3> LIST=<1:TRUE,2:TRUE>"
	SIM_dialog_add_item	"SYS_POSITION_ITEM		S3 POSITION=<AXES> AXES_LIST=<Z1,X1,TURRET1,Z2,X2,TURRET2>"
	SIM_dialog_add_item	"SYS_MESSAGE_WINDOW_ITEM	S4"
	SIM_dialog_add_item	"SYS_STEP_CONTROL_ITEM		S5"
	SIM_dialog_add_item	"SYS_ANIMATION_SPEED_ITEM	S6"
	
      SIM_dialog_end

     # Start of Channel Window Dialog
      SIM_dialog_start		"SYS_CHANNEL_WINDOW TITLE=<Channel Window>"
	
       # 1st Channel
	SIM_dialog_add_item	"SYS_START_CHANNEL_GROUP      1"
	SIM_dialog_add_item	"SYS_ACTIVE_TOOL_ITEM         U1"
	SIM_dialog_add_item	"SYS_COOLANT_ITEM             U2 ATTACHMENT=<LEFT 0 10>"
	SIM_dialog_add_item	"SYS_SPEED_ITEM               U3"
	SIM_dialog_add_item	"SYS_FEED_ITEM                U4 ATTACHMENT=<LEFT 0 10>"
	SIM_dialog_add_item	"SYS_SPINDLE_SPEED_ITEM       U5"
	SIM_dialog_add_item	"SYS_SURFACE_SPEED_ITEM       U6 ATTACHMENT=<LEFT 0 5>"
	SIM_dialog_add_item	"SYS_MAX_SPINDLE_SPEED_ITEM   U7"
	SIM_dialog_add_item	"SYS_TOOL_POSITION_ITEM       U8"
	SIM_dialog_add_item	"SYS_PROGRAM_WINDOW_ITEM      U9 SPAN=<4> WIDTH=<33>"
	SIM_dialog_add_item	"USR_SEPARATOR_ITEM           SP1 ORIENTATION=<VERTICAL> HEIGHT=<198> ATTACHMENT=<LEFT 0 0>"
	
       # 2nd Channel
	SIM_dialog_add_item	"SYS_START_CHANNEL_GROUP      2"
	SIM_dialog_add_item	"SYS_ACTIVE_TOOL_ITEM         U1"
	SIM_dialog_add_item	"SYS_COOLANT_ITEM             U2 ATTACHMENT=<LEFT 0 10>"
	SIM_dialog_add_item	"SYS_SPEED_ITEM               U3"
	SIM_dialog_add_item	"SYS_FEED_ITEM                U4 ATTACHMENT=<LEFT 0 10>"
	SIM_dialog_add_item	"SYS_SPINDLE_SPEED_ITEM       U5"
	SIM_dialog_add_item	"SYS_SURFACE_SPEED_ITEM       U6 ATTACHMENT=<LEFT 0 5>"
	SIM_dialog_add_item	"SYS_MAX_SPINDLE_SPEED_ITEM   U7"
	SIM_dialog_add_item	"SYS_TOOL_POSITION_ITEM       U8"
	SIM_dialog_add_item	"SYS_PROGRAM_WINDOW_ITEM      U9 SPAN=<4> WIDTH=<33>"
	
     # End of Channel Window Dialog
      SIM_dialog_end
   }
}


#=============================================================
proc PB_CMD_vnc__create_tmp_jct { } {
#=============================================================
  global mom_sim_tool_junction mom_sim_current_junction
  global mom_sim_curr_jct_matrix mom_sim_curr_jct_origin
  global mom_sim_tool_loaded


   if { ![string match "$mom_sim_tool_junction" "$mom_sim_current_junction"] } {

     # Somehow junctions can not be deleted!?
      global mom_sim_result
      global mom_sim_tmp_tool_jct_index

      if { ![info exists mom_sim_tmp_tool_jct_index] } {
         set mom_sim_tmp_tool_jct_index 0
      }

      SIM_ask_is_junction_exist __SIM_TMP_TOOL_JCT__$mom_sim_tmp_tool_jct_index

      if { $mom_sim_result == 1 } {
         incr mom_sim_tmp_tool_jct_index
      }
      set tmp_tool_jct "__SIM_TMP_TOOL_JCT__$mom_sim_tmp_tool_jct_index"

      eval SIM_create_junction $tmp_tool_jct $mom_sim_tool_loaded  $mom_sim_curr_jct_origin $mom_sim_curr_jct_matrix

      SIM_set_current_ref_junction $tmp_tool_jct

      set mom_sim_tool_junction $tmp_tool_jct
      set mom_sim_current_junction $mom_sim_tool_junction

     # Update mom_sim_pos w.r.t new ref jct
      if { ![catch {set pos [SIM_ask_last_position_zcs]} ] } {
         global mom_sim_pos
         set mom_sim_pos(0) [lindex $pos 0]
         set mom_sim_pos(1) [lindex $pos 1]
         set mom_sim_pos(2) [lindex $pos 2]
      }
   }
}


#=============================================================
proc PB_CMD_vnc__cycle_move { } {
#=============================================================
  global mom_sim_lg_axis
  global mom_sim_pos mom_sim_prev_pos
  global mom_sim_cycle_rapid_to_pos
  global mom_sim_cycle_feed_to_pos
  global mom_sim_cycle_retract_to_pos
  global mom_sim_cycle_mode
  global mom_sim_nc_register


   set mom_sim_pos(0) $mom_sim_prev_pos(0)
   set mom_sim_pos(1) $mom_sim_prev_pos(1)
   set mom_sim_pos(2) $mom_sim_prev_pos(2)


   if $mom_sim_cycle_mode(start_block) {

     global mom_sim_cycle_spindle_axis
     global mom_sim_cycle_rapid_to_dist
     global mom_sim_cycle_feed_to_dist
     global mom_sim_cycle_retract_to_dist
     global mom_sim_cycle_entry_pos

      set rapid_to_pos(0) $mom_sim_nc_register(X)
      set rapid_to_pos(1) $mom_sim_nc_register(Y)
      set rapid_to_pos(2) $mom_sim_nc_register(Z)

      if [expr $mom_sim_cycle_entry_pos($mom_sim_cycle_spindle_axis) > $rapid_to_pos($mom_sim_cycle_spindle_axis)] {
         set mom_sim_cycle_entry_pos($mom_sim_cycle_spindle_axis) $rapid_to_pos($mom_sim_cycle_spindle_axis)
      }

      set rapid_to_pos($mom_sim_cycle_spindle_axis)  $mom_sim_cycle_rapid_to_pos($mom_sim_cycle_spindle_axis)

      if [string match "*R -*" $mom_sim_cycle_mode(rapid_to)] {
         if [string match "*Distance*" $mom_sim_cycle_mode(rapid_to)] {
            set rapid_to_pos($mom_sim_cycle_spindle_axis) [expr $mom_sim_cycle_entry_pos($mom_sim_cycle_spindle_axis) + $mom_sim_cycle_rapid_to_dist]
         }
      }
      set mom_sim_cycle_rapid_to_pos(0) $rapid_to_pos(0)
      set mom_sim_cycle_rapid_to_pos(1) $rapid_to_pos(1)
      set mom_sim_cycle_rapid_to_pos(2) $rapid_to_pos(2)

      set feed_to_pos(0) $mom_sim_nc_register(X)
      set feed_to_pos(1) $mom_sim_nc_register(Y)
      set feed_to_pos(2) $mom_sim_nc_register(Z)
      set feed_to_pos($mom_sim_cycle_spindle_axis)  $mom_sim_cycle_feed_to_pos($mom_sim_cycle_spindle_axis)

      switch $mom_sim_cycle_mode(feed_to) {
         "Distance" {
            set feed_to_pos($mom_sim_cycle_spindle_axis) [expr $mom_sim_cycle_entry_pos($mom_sim_cycle_spindle_axis) - $mom_sim_cycle_feed_to_dist]
         }
      }
      set mom_sim_cycle_feed_to_pos(0) $feed_to_pos(0)
      set mom_sim_cycle_feed_to_pos(1) $feed_to_pos(1)
      set mom_sim_cycle_feed_to_pos(2) $feed_to_pos(2)

      set retract_to_pos(0) $mom_sim_nc_register(X)
      set retract_to_pos(1) $mom_sim_nc_register(Y)
      set retract_to_pos(2) $mom_sim_nc_register(Z)
      set retract_to_pos($mom_sim_cycle_spindle_axis)  $mom_sim_cycle_retract_to_pos($mom_sim_cycle_spindle_axis)

      if [string match "K -*" $mom_sim_cycle_mode(retract_to)] {
         if [string match "*Distance*" $mom_sim_cycle_mode(retract_to)] {
            set retract_to_pos($mom_sim_cycle_spindle_axis) [expr $mom_sim_cycle_entry_pos($mom_sim_cycle_spindle_axis) + $mom_sim_cycle_retract_to_dist]
         }
      }
      set mom_sim_cycle_retract_to_pos(0) $retract_to_pos(0)
      set mom_sim_cycle_retract_to_pos(1) $retract_to_pos(1)
      set mom_sim_cycle_retract_to_pos(2) $retract_to_pos(2)
   }


   for { set i 0 } { $i < 3 } { incr i } {
      set sim_pos_save($i) $mom_sim_pos($i)
   }
   set sim_motion_type_save $mom_sim_nc_register(MOTION)

   set mom_sim_nc_register(MOTION) LINEAR
   set mom_sim_pos(0) $mom_sim_cycle_rapid_to_pos(0)
   set mom_sim_pos(1) $mom_sim_cycle_rapid_to_pos(1)
   set mom_sim_pos(2) $mom_sim_cycle_rapid_to_pos(2)

   PB_CMD_vnc__sim_motion

   set mom_sim_pos(0) $mom_sim_cycle_feed_to_pos(0)
   set mom_sim_pos(1) $mom_sim_cycle_feed_to_pos(1)
   set mom_sim_pos(2) $mom_sim_cycle_feed_to_pos(2)

   PB_CMD_vnc__sim_motion

   set mom_sim_pos(0) $mom_sim_cycle_retract_to_pos(0)
   set mom_sim_pos(1) $mom_sim_cycle_retract_to_pos(1)
   set mom_sim_pos(2) $mom_sim_cycle_retract_to_pos(2)

   PB_CMD_vnc__sim_motion

  # Restore data
   for { set i 0 } { $i < 3 } { incr i } {
      set mom_sim_pos($i) $sim_pos_save($i)
   }
   set mom_sim_nc_register(MOTION) $sim_motion_type_save
}


#=============================================================
proc PB_CMD_vnc__cycle_set { } {
#=============================================================
  global mom_sim_o_buffer
  global mom_sim_address mom_sim_pos mom_sim_prev_pos
  global mom_sim_nc_register
  global mom_sim_cycle_mode
  global mom_sim_cycle_rapid_to_pos
  global mom_sim_cycle_feed_to_pos
  global mom_sim_cycle_retract_to_pos
  global mom_sim_cycle_entry_pos
  global mom_sim_cycle_rapid_to_dist
  global mom_sim_cycle_feed_to_dist
  global mom_sim_cycle_retract_to_dist
  global mom_sim_cycle_spindle_axis

  global mom_sim_machine_type
  global mom_sim_4th_axis_plane mom_sim_5th_axis_plane


  # Interrogate tool axis to determine whether principal axis should be reversed.
  # This direction is needed when any of the cycle's parmeters is incremental.

   set axis_direction 1

   if [string match "*head*" $mom_sim_machine_type] {
      switch $mom_sim_4th_axis_plane {
         "ZX" {
            if [EQ_is_equal $mom_sim_nc_register(fourth_axis) -90.0] {
               set axis_direction -1
            }
         }
         "YZ" {
            if [EQ_is_equal $mom_sim_nc_register(fourth_axis) 90.0] {
               set axis_direction -1
            }
         }
      }
   }

   if [string match "5_axis_dual_head" $mom_sim_machine_type] {
      switch $mom_sim_5th_axis_plane {
         "ZX" {
            if [EQ_is_equal $mom_sim_nc_register(fifth_axis) -90.0] {
               set axis_direction -1
            }
         }
         "YZ" {
            if [EQ_is_equal $mom_sim_nc_register(fifth_axis) 90.0] {
               set axis_direction -1
            }
         }
      }
   }


   set mom_sim_pos(0) $mom_sim_prev_pos(0)
   set mom_sim_pos(1) $mom_sim_prev_pos(1)
   set mom_sim_pos(2) $mom_sim_prev_pos(2)

   set mom_sim_cycle_rapid_to_pos($mom_sim_cycle_spindle_axis) $mom_sim_pos($mom_sim_cycle_spindle_axis)

   set mom_sim_cycle_entry_pos($mom_sim_cycle_spindle_axis) $mom_sim_pos($mom_sim_cycle_spindle_axis)


  # nc_register(K_cycle) may never be set, since it's in conflict with nc_register(K).
   if [info exists mom_sim_nc_register(K)] {
      set mom_sim_nc_register(K_cycle) $mom_sim_nc_register(K)
   }

   if { ![info exists mom_sim_nc_register(K_cycle)] } {
      set mom_sim_nc_register(K_cycle) 0
   }

   if { ![info exists mom_sim_nc_register(R)] } {
      set mom_sim_nc_register(R) 0
   }


global mom_sim_message mom_sim_cycle_feed_to_addr
set mom_sim_message "CYCLE : $mom_sim_nc_register($mom_sim_cycle_feed_to_addr)  $mom_sim_nc_register(R) $mom_sim_nc_register(K_cycle)"
PB_CMD_vnc__send_message


   set mom_sim_cycle_feed_to_pos(0) $mom_sim_nc_register(X)
   set mom_sim_cycle_feed_to_pos(1) $mom_sim_nc_register(Y)
   set mom_sim_cycle_feed_to_pos(2) $mom_sim_nc_register(Z)

   switch $mom_sim_cycle_mode(feed_to) {
      "Distance" {
         set mom_sim_cycle_feed_to_dist [expr abs($mom_sim_nc_register(Z)) * $axis_direction]

         if { $mom_sim_cycle_spindle_axis == 2 } { ;# Z axis
            set mom_sim_cycle_feed_to_pos(2) [expr $mom_sim_pos(2) - $mom_sim_cycle_feed_to_dist]
         } else {
            set mom_sim_cycle_feed_to_pos(2) $mom_sim_pos(2)
         }

         set mom_sim_cycle_entry_pos($mom_sim_cycle_spindle_axis) [expr $mom_sim_cycle_feed_to_pos($mom_sim_cycle_spindle_axis) + $mom_sim_cycle_feed_to_dist]
      }
   }


   set mom_sim_cycle_rapid_to_pos(0) $mom_sim_cycle_feed_to_pos(0)
   set mom_sim_cycle_rapid_to_pos(1) $mom_sim_cycle_feed_to_pos(1)
   set mom_sim_cycle_rapid_to_pos(2) $mom_sim_cycle_feed_to_pos(2)

   if [string match "*R -*" $mom_sim_cycle_mode(rapid_to)] {
      if [string match "*Distance*" $mom_sim_cycle_mode(rapid_to)] {
#>>>
        # The 1st hole may have been @ rapid-to pos already.
         set mom_sim_cycle_rapid_to_dist [expr abs($mom_sim_nc_register(R)) * $axis_direction]

         if [expr $mom_sim_cycle_entry_pos($mom_sim_cycle_spindle_axis) ==  $mom_sim_pos($mom_sim_cycle_spindle_axis)] {
            set mom_sim_cycle_entry_pos($mom_sim_cycle_spindle_axis) [expr $mom_sim_cycle_entry_pos($mom_sim_cycle_spindle_axis) - $mom_sim_cycle_rapid_to_dist]
         }

         set mom_sim_cycle_rapid_to_pos($mom_sim_cycle_spindle_axis) [expr $mom_sim_cycle_entry_pos($mom_sim_cycle_spindle_axis) + $mom_sim_cycle_rapid_to_dist]

      } else {
         set mom_sim_cycle_rapid_to_pos($mom_sim_cycle_spindle_axis) $mom_sim_nc_register(R)
      }
   }


   set mom_sim_cycle_retract_to_pos(0) $mom_sim_cycle_rapid_to_pos(0)
   set mom_sim_cycle_retract_to_pos(1) $mom_sim_cycle_rapid_to_pos(1)
   set mom_sim_cycle_retract_to_pos(2) $mom_sim_cycle_rapid_to_pos(2)

   if [string match "K*" $mom_sim_cycle_mode(retract_to)] {
      if [string match "*Distance*" $mom_sim_cycle_mode(retract_to)] {
#>>>
        # The 1st hole may have been @ rapid-to pos already.
         set mom_sim_cycle_retract_to_dist [expr abs($mom_sim_nc_register(K_cycle)) * $axis_direction]

         if [expr $mom_sim_cycle_entry_pos($mom_sim_cycle_spindle_axis) ==  $mom_sim_pos($mom_sim_cycle_spindle_axis)] {
            set mom_sim_cycle_entry_pos($mom_sim_cycle_spindle_axis) [expr $mom_sim_cycle_entry_pos($mom_sim_cycle_spindle_axis) - $mom_sim_cycle_retract_to_dist]
         }

         set mom_sim_cycle_retract_to_pos($mom_sim_cycle_spindle_axis) [expr $mom_sim_cycle_entry_pos($mom_sim_cycle_spindle_axis) + $mom_sim_cycle_retract_to_dist]
      } else {
         set mom_sim_cycle_retract_to_pos($mom_sim_cycle_spindle_axis) $mom_sim_nc_register(K_cycle)
      }

   } elseif [string match "G98/G99*" $mom_sim_cycle_mode(retract_to)] {

      global mom_sim_nc_func

      if { [VNC_parse_nc_block mom_sim_o_buffer $mom_sim_nc_func(CYCLE_RETURN_MANUAL)] == 1 } {
         switch $mom_sim_cycle_spindle_axis {
            0 {
               set mom_sim_cycle_retract_to_pos($mom_sim_cycle_spindle_axis) $mom_sim_nc_register(LAST_X)
            }
            1 {
               set mom_sim_cycle_retract_to_pos($mom_sim_cycle_spindle_axis) $mom_sim_nc_register(LAST_Y)
            }
            default {
               set mom_sim_cycle_retract_to_pos($mom_sim_cycle_spindle_axis) $mom_sim_nc_register(LAST_Z)
            }
         }
      }
   }


   if { !$mom_sim_cycle_mode(start_block) } {
      PB_CMD_vnc__cycle_move
   }
}


#=============================================================
proc PB_CMD_vnc__display_version { } {
#=============================================================
  global mom_sim_vnc_handler mom_sim_post_builder_version

   PB_VNC_send_message "> Virtual NC Controller ($mom_sim_vnc_handler) created with Post Builder v$mom_sim_post_builder_version."
}


#=============================================================
proc PB_CMD_vnc__end_of_path { } {
#=============================================================
}


#=============================================================
proc PB_CMD_vnc__end_of_program { } {
#=============================================================
  global mom_sim_csys_set

   set mom_sim_csys_set 0
}


#=============================================================
proc PB_CMD_vnc__fix_nc_definitions { } {
#=============================================================
  global mom_sim_nc_func

  # Added these G codes here to force v321 file conversion
  # for the changes in default PB_CMD_vnc____set_nc_definitions
  #
   set mom_sim_nc_func(RETURN_HOME_P)       "G30"
   set mom_sim_nc_func(FROM_HOME)           "G29"
   set mom_sim_nc_func(MACH_CS_MOVE)        "G53"
   set mom_sim_nc_func(WORK_CS_1)           "G54"
   set mom_sim_nc_func(WORK_CS_2)           "G55"
   set mom_sim_nc_func(WORK_CS_3)           "G56"
   set mom_sim_nc_func(WORK_CS_4)           "G57"
   set mom_sim_nc_func(WORK_CS_5)           "G58"
   set mom_sim_nc_func(WORK_CS_6)           "G59"
   set mom_sim_nc_func(LOCAL_CS_SET)        "G52"

  # Set circular plane code for legacy VNC
   global mom_sim_circular_plane
   set mom_sim_circular_plane XY
}


#=============================================================
proc PB_CMD_vnc__from_home { } {
#=============================================================
  global mom_sim_nc_register
  global mom_sim_pos
  global mom_sim_lg_axis


  # Move to the intermediate pt
   set coord_list [list]
   if [info exists mom_sim_nc_register(REF_INT_PT_X_PREV)] {
      lappend coord_list $mom_sim_lg_axis(X)  $mom_sim_nc_register(REF_INT_PT_X_PREV)
   }
   if [info exists mom_sim_nc_register(REF_INT_PT_Y_PREV)] {
      lappend coord_list $mom_sim_lg_axis(Y)  $mom_sim_nc_register(REF_INT_PT_Y_PREV)
   }
   if [info exists mom_sim_nc_register(REF_INT_PT_Z_PREV)] {
      lappend coord_list $mom_sim_lg_axis(Z)  $mom_sim_nc_register(REF_INT_PT_Z_PREV)
   }

   global mom_sim_num_machine_axes
   if { $mom_sim_num_machine_axes > 3 } {
      if [info exists mom_sim_nc_register(REF_INT_PT_4_PREV)] {
         lappend coord_list $mom_sim_lg_axis(4)  $mom_sim_nc_register(REF_INT_PT_4_PREV)
      }
   }

   if { $mom_sim_num_machine_axes > 4 } {
      if [info exists mom_sim_nc_register(REF_INT_PT_5_PREV)] {
         lappend coord_list $mom_sim_lg_axis(5)  $mom_sim_nc_register(REF_INT_PT_5_PREV)
      }
   }

  # Create a temp jct for subsequent moves incase the tool is not set properly.
   VNC_create_tmp_jct


  # Move to the intermediate point.
   if { [llength $coord_list] > 0 } {
      eval VNC_move_linear_zcs RAPID $coord_list
   }

  # Move to target
   switch $mom_sim_num_machine_axes {
      4 {
         VNC_move_linear_zcs RAPID  $mom_sim_lg_axis(X) $mom_sim_pos(0) $mom_sim_lg_axis(Y) $mom_sim_pos(1) $mom_sim_lg_axis(Z) $mom_sim_pos(2) $mom_sim_lg_axis(4) $mom_sim_pos(3)
      }
      5 {
         VNC_move_linear_zcs RAPID  $mom_sim_lg_axis(X) $mom_sim_pos(0) $mom_sim_lg_axis(Y) $mom_sim_pos(1) $mom_sim_lg_axis(Z) $mom_sim_pos(2) $mom_sim_lg_axis(4) $mom_sim_pos(3) $mom_sim_lg_axis(5) $mom_sim_pos(4)
      }
      default {
         VNC_move_linear_zcs RAPID  $mom_sim_lg_axis(X) $mom_sim_pos(0) $mom_sim_lg_axis(Y) $mom_sim_pos(1) $mom_sim_lg_axis(Z) $mom_sim_pos(2)
      }
   }
}


#=============================================================
proc PB_CMD_vnc__init_sim_vars { } {
#=============================================================
  global mom_sim_word_separator mom_sim_word_separator_len
  global mom_sim_end_of_block mom_sim_end_of_block_len
  global mom_sim_pos
  global mom_sim_tool_loaded
  global mom_sim_tool_junction
  global mom_sim_curr_jct_matrix mom_sim_curr_jct_origin
  global mom_sim_nc_register
  global mom_sim_wcs_offsets
  global mom_sim_ug_tool_name
  global mom_sim_spindle_comp mom_sim_spindle_jct
  global mom_sim_mt_axis
  global mom_sim_PI
  global mom_sim_cycle_spindle_axis
  global mom_sim_vnc_msg_prefix


   set mom_sim_PI [expr acos(-1.0)]

   set mom_sim_vnc_msg_prefix "VNC_MSG::"


  # Initialize a null list for other NC codes to be simulated.
  # Actual code list will be defined in PB_CMD_vnc____set_nc_definitions.
  #
   global mom_sim_other_nc_codes
   set mom_sim_other_nc_codes [list]


  # Initialize physical machine tool axes assignment
  # So there won't be null variable referenced in subsequent code.
  # Actual assignments are carried out in ____map function.
  #
   set mom_sim_mt_axis(X) X
   set mom_sim_mt_axis(Y) Y
   set mom_sim_mt_axis(Z) Z
   set mom_sim_mt_axis(4) A
   set mom_sim_mt_axis(5) B


  # Initialize cycle spindle axis to Z
  #
   set mom_sim_cycle_spindle_axis 2


  # Transfer static data from the Post.
  #
   VNC_set_nc_definitions



  # When a VNC is used to output VNC messages only w/o simulation.
  #
   global mom_sim_vnc_msg_only
   if { [info exists mom_sim_vnc_msg_only] && $mom_sim_vnc_msg_only } {
return
   }



  # Display message about this VNC.
  #
   if [llength [info commands "PB_CMD_vnc__display_version"]] {
      PB_CMD_vnc__display_version
   }


  # Add user's NC data definition.
  #
   if [llength [info commands "PB_CMD_vnc__set_nc_definitions"]] {
      PB_CMD_vnc__set_nc_definitions
   } elseif [llength [info commands "PB_CMD_vnc____set_nc_definitions"]] {
      PB_CMD_vnc____set_nc_definitions
   }


  # Add more user's NC data definition required for conversion.
  #
   if [llength [info commands "PB_CMD_vnc__fix_nc_definitions"]] {
      PB_CMD_vnc__fix_nc_definitions
   }


  # Map machine tool axes assignments
  #
   PB_CMD_vnc____map_machine_tool_axes


  # Nullify ref pos of MTCS
   global mom_sim_pos_mtcs
   if [info exists mom_sim_pos_mtcs(0)] {
      unset mom_sim_pos_mtcs(0)
   }
   if [info exists mom_sim_pos_mtcs(1)] {
      unset mom_sim_pos_mtcs(1)
   }
   if [info exists mom_sim_pos_mtcs(2)] {
      unset mom_sim_pos_mtcs(2)
   }


  # Logical axes assignments (not from the post?)
  # It works to equate logical names to the physical names.
  #
   global mom_sim_lg_axis mom_sim_nc_axes mom_sim_address
   global mom_sim_num_machine_axes

  # Linear axes are commanded by logical names, whereas
  # rotary axes are commanded by physical names.
  #
   set mom_sim_lg_axis(X) $mom_sim_address(X,leader)
   set mom_sim_lg_axis(Y) $mom_sim_address(Y,leader)
   set mom_sim_lg_axis(Z) $mom_sim_address(Z,leader)

   set mom_sim_nc_axes [concat $mom_sim_lg_axis(X) $mom_sim_lg_axis(Y) $mom_sim_lg_axis(Z)]

   switch $mom_sim_num_machine_axes {
      4 {
         set mom_sim_lg_axis(4) $mom_sim_mt_axis(4)
         lappend mom_sim_nc_axes $mom_sim_lg_axis(4)
      }
      5 {
         set mom_sim_lg_axis(4) $mom_sim_mt_axis(4)
         set mom_sim_lg_axis(5) $mom_sim_mt_axis(5)
         lappend mom_sim_nc_axes $mom_sim_lg_axis(4) $mom_sim_lg_axis(5)
      }
   }


  # Initialize data for simulation.
  #
   if { ![info exists mom_sim_ug_tool_name] } {
      set mom_sim_ug_tool_name ""
   }

   if { ![info exists mom_sim_tool_loaded] } {
      set mom_sim_tool_loaded ""
   }

   if { ![info exists mom_sim_tool_junction] } {
      set mom_sim_tool_junction "$mom_sim_spindle_jct"
   }

   if { ![info exists mom_sim_curr_jct_matrix] } {
      lappend mom_sim_curr_jct_matrix 1.0 0.0 0.0  0.0 1.0 0.0  0.0 0.0 1.0
   }

   if { ![info exists mom_sim_curr_jct_origin] } {
      lappend mom_sim_curr_jct_origin 0.0 0.0 0.0
   }

   for {set i 0} {$i < 8} {incr i} {
      if { ![info exists mom_sim_pos($i)] } {
        # set mom_sim_pos($i) ""
         set mom_sim_pos($i) 0
      }
   }


  # Initialize controller state
  #
   global mom_sim_output_unit
   if { ![info exists mom_sim_nc_register(UNIT)] } {
      set mom_sim_nc_register(UNIT) $mom_sim_output_unit
   }
   if { ![info exists mom_sim_nc_register(MOTION)] } {
      set mom_sim_nc_register(MOTION) RAPID
   }
   if { ![info exists mom_sim_nc_register(INPUT)] } {
      set mom_sim_nc_register(INPUT) ABS
   }
   if { ![info exists mom_sim_nc_register(STROKE_LIMIT)] } {
      set mom_sim_nc_register(STROKE_LIMIT) ON
   }
   if { ![info exists mom_sim_nc_register(CUTCOM)] } {
      set mom_sim_nc_register(CUTCOM) OFF
   }
   if { ![info exists mom_sim_nc_register(TL_ADJUST)] } {
      set mom_sim_nc_register(TL_ADJUST) OFF
   }
   if { ![info exists mom_sim_nc_register(SCALE)] } {
      set mom_sim_nc_register(SCALE) OFF
   }
   if { ![info exists mom_sim_nc_register(WCS)] } {
      set mom_sim_nc_register(WCS) 1
   }
   if { ![info exists mom_sim_nc_register(MACRO_MODAL)] } {
      set mom_sim_nc_register(MACRO_MODAL) OFF
   }
   if { ![info exists mom_sim_nc_register(WCS_ROTATE)] } {
      set mom_sim_nc_register(WCS_ROTATE) OFF
   }
   if { ![info exists mom_sim_nc_register(CYCLE)] } {
      set mom_sim_nc_register(CYCLE) OFF
   }
   if { ![info exists mom_sim_nc_register(CYCLE_RETURN)] } {
      set mom_sim_nc_register(CYCLE_RETURN) INIT_LEVEL
   }
   if { ![info exists mom_sim_nc_register(RETURN_HOME)] } {
      set mom_sim_nc_register(RETURN_HOME) 0
   }
   if { ![info exists mom_sim_nc_register(FROM_HOME)] } {
      set mom_sim_nc_register(FROM_HOME) 0
   }
   if { ![info exists mom_sim_nc_register(LOCAL_OFFSET)] } {
      set mom_sim_nc_register(LOCAL_OFFSET) [list]
      lappend mom_sim_nc_register(LOCAL_OFFSET) 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0
   }
   if { ![info exists mom_sim_nc_register(WORK_OFFSET)] } {
      set mom_sim_nc_register(WORK_OFFSET) [list]
      foreach coord $mom_sim_wcs_offsets($mom_sim_nc_register(WCS)) {
         lappend mom_sim_nc_register(WORK_OFFSET) $coord
      }
   }

   if { ![info exists mom_sim_nc_register(SPINDLE_DIRECTION)] } {
      set mom_sim_nc_register(SPINDLE_DIRECTION) OFF
   }
   if { ![info exists mom_sim_nc_register(SPINDLE_MODE)] } {
      set mom_sim_nc_register(SPINDLE_MODE) REV_PER_MIN
   }
   if { ![info exists mom_sim_nc_register(S)] } {
      set mom_sim_nc_register(S) 0
   }
   if { ![info exists mom_sim_nc_register(FEED_MODE)] } {
      switch $mom_sim_nc_register(UNIT) {
         MM {
            set mom_sim_nc_register(FEED_MODE) MM_PER_MIN
         }
         default {
            set mom_sim_nc_register(FEED_MODE) INCH_PER_MIN
         }
      }
   }
   if { ![info exists mom_sim_nc_register(F)] } {
      set mom_sim_nc_register(F) 0
   }


#<09-01-04 gsl>
  # Initialize plane code
  #
   if { ![info exists mom_sim_nc_register(PLANE)] } {
      set mom_sim_nc_register(PLANE) XY
   }


#>>>>>
  # Initialize subspindle activation state
  #
   if { ![info exists mom_sim_nc_register(CROSS_MACHINING)] } {
      set mom_sim_nc_register(CROSS_MACHINING) 0
   }

  # Initialize machine mode
  #
   global mom_sim_machine_type
   if [string match "*wedm*" $mom_sim_machine_type] {
      set mom_sim_nc_register(MACHINE_MODE) WEDM
   } elseif [string match "*lathe*" $mom_sim_machine_type] {
      set mom_sim_nc_register(MACHINE_MODE) TURN
   } else {
      set mom_sim_nc_register(MACHINE_MODE) MILL
   }

  # Initialize CSYS state
  #
   global mom_sim_csys_set
   set mom_sim_csys_set 0

  # Initialize tool offsets
  #
   global mom_sim_tool_offset_used mom_sim_tool_offset

   if { ![info exists mom_sim_tool_offset_used] } {
      set mom_sim_tool_offset_used 0
   }

   if { ![info exists mom_sim_tool_offset] } {
      set mom_sim_tool_offset(0) 0.0
      set mom_sim_tool_offset(1) 0.0
      set mom_sim_tool_offset(2) 0.0
   }

   global mom_sim_pivot_distance
   if { ![info exists mom_sim_pivot_distance] } {
      set mom_sim_pivot_distance 0.0
   }

  # Set default N/C output unit.  This value will be used
  # when no output unit function is specified in the N/C code.
  # It's also used as the intial mode when controller is reset.
  #
   set mom_sim_nc_register(DEF_UNIT)       $mom_sim_nc_register(UNIT)

  #-----------------------------------
  # Set the machine tool driver units.
  #-----------------------------------
   if { $mom_sim_nc_register(UNIT) == "MM" } {
      SIM_set_mtd_units "MM" 
   } else {
      SIM_set_mtd_units "INCH" 
   }

  #--------------------------------
  # Set rotary axis direction mode.
  #--------------------------------
   global mom_sim_4th_axis_direction mom_sim_5th_axis_direction
   global mom_sim_4th_axis_max_limit mom_sim_5th_axis_max_limit
   global mom_sim_4th_axis_min_limit mom_sim_5th_axis_min_limit

   if [info exists mom_sim_lg_axis(4)] {
      if { ![string match " " $mom_sim_4th_axis_direction] } {
         if [string match "MAGNITUDE_DETERMINES_DIRECTION" $mom_sim_4th_axis_direction] {
            if { [expr $mom_sim_4th_axis_min_limit == 0.0] &&  [expr $mom_sim_4th_axis_max_limit == 360.0] } {
               set mom_sim_4th_axis_direction "ALWAYS_SHORTEST"
            }
         }
         SIM_set_axis_rotary_dir_mode $mom_sim_lg_axis(4) $mom_sim_4th_axis_direction
      }
   }
   if [info exists mom_sim_lg_axis(5)] {
      if { ![string match " " $mom_sim_5th_axis_direction] } {
         if [string match "MAGNITUDE_DETERMINES_DIRECTION" $mom_sim_5th_axis_direction] {
            if { [expr $mom_sim_5th_axis_min_limit == 0.0] &&  [expr $mom_sim_5th_axis_max_limit == 360.0] } {
               set mom_sim_5th_axis_direction "ALWAYS_SHORTEST"
            }
         }
         SIM_set_axis_rotary_dir_mode $mom_sim_lg_axis(5) $mom_sim_5th_axis_direction
      }
   }
}


#=============================================================
proc PB_CMD_vnc__init_sync_manager { } {
#=============================================================
  global mom_sim_vnc_msg_only
  global mom_sim_primary_channel

   if { ![info exists mom_sim_vnc_msg_only] || !$mom_sim_vnc_msg_only } {

      SIM_mach_max_axis_rapid_velocity X1 20 M_PER_MIN
      SIM_mach_max_axis_rapid_velocity Y 20 M_PER_MIN
      SIM_mach_max_axis_rapid_velocity B 20 M_PER_MIN
      SIM_mach_max_axis_rapid_velocity Z1 20 M_PER_MIN
      SIM_mach_max_axis_rapid_velocity X2 20 M_PER_MIN
      SIM_mach_max_axis_rapid_velocity Z2 20 M_PER_MIN
      SIM_mach_max_axis_rapid_velocity TURRET 20 M_PER_MIN

      SIM_mach_max_axis_rapid_velocity C1 10 REV_PER_MIN
      SIM_mach_max_axis_rapid_velocity C2 10 REV_PER_MIN

      SIM_mach_max_spindle_speed CHUCK1 8000 REV_PER_MIN
      SIM_mach_max_spindle_speed CHUCK2 4500 REV_PER_MIN

     # By default all channele assigned to spindle one
      SIM_mach_assign_channel_to_spindle 1 CHUCK1
      SIM_mach_assign_channel_to_spindle 2 CHUCK1

      SIM_mach_default_primary_channel 1
      SIM_mach_default_primary_channel 2

      set mom_sim_primary_channel 1
   }
}


#=============================================================
proc PB_CMD_vnc__linear_move { } {
#=============================================================
  global mom_sim_lg_axis mom_sim_pos
  global mom_sim_nc_register
  global mom_sim_num_machine_axes


   switch $mom_sim_num_machine_axes {
      4 {
         VNC_move_linear_zcs CUT  $mom_sim_lg_axis(X) $mom_sim_pos(0)  $mom_sim_lg_axis(Y) $mom_sim_pos(1)  $mom_sim_lg_axis(Z) $mom_sim_pos(2)  $mom_sim_lg_axis(4) $mom_sim_pos(3)
      }

      5 {
         VNC_move_linear_zcs CUT  $mom_sim_lg_axis(X) $mom_sim_pos(0)  $mom_sim_lg_axis(Y) $mom_sim_pos(1)  $mom_sim_lg_axis(Z) $mom_sim_pos(2)  $mom_sim_lg_axis(4) $mom_sim_pos(3)  $mom_sim_lg_axis(5) $mom_sim_pos(4)
      }
      default {
         VNC_move_linear_zcs CUT  $mom_sim_lg_axis(X) $mom_sim_pos(0)  $mom_sim_lg_axis(Y) $mom_sim_pos(1)  $mom_sim_lg_axis(Z) $mom_sim_pos(2)
      }
   }

   set mom_sim_nc_register(LAST_X) $mom_sim_pos(0)
   set mom_sim_nc_register(LAST_Y) $mom_sim_pos(1)
   set mom_sim_nc_register(LAST_Z) $mom_sim_pos(2)
}


#=============================================================
proc PB_CMD_vnc__mount_part { } {
#=============================================================
  # Comment out next statement to snap part to the sub spindle.
return

   SIM_mount_kim_comp 10 PART_SUB PART_SUB_MOUNT_JCT CHUCK2 CHUCK2_JCT
   SIM_update
}


#=============================================================
proc PB_CMD_vnc__nurbs_move { } {
#=============================================================
   global mom_sim_nurbs_order
   global mom_sim_nurbs_knot_count
   global mom_sim_nurbs_knots
   global mom_sim_nurbs_point_count
   global mom_sim_nurbs_points

   SIM_move_nurbs_zcs $mom_sim_nurbs_point_count $mom_sim_nurbs_order  $mom_sim_nurbs_knot_count $mom_sim_nurbs_knots $mom_sim_nurbs_points
}


#=============================================================
proc PB_CMD_vnc__offset_tool_jct { } {
#=============================================================
# Offsets supplied in Tool Junction coordinate
#
  global mom_sim_tool_x_offset mom_sim_tool_y_offset mom_sim_tool_z_offset

  global mom_sim_current_junction
  global mom_sim_tool_junction
  global mom_sim_curr_jct_matrix mom_sim_curr_jct_origin
  global mom_sim_result mom_sim_result1


   if { ![expr (abs($mom_sim_tool_x_offset) +  abs($mom_sim_tool_y_offset) +  abs($mom_sim_tool_z_offset)) == 0.0] } {


global mom_sim_message
set mom_sim_message "**TOOL JCT OFFSET: $mom_sim_tool_x_offset $mom_sim_tool_y_offset $mom_sim_tool_z_offset"
PB_CMD_vnc__send_message


      SIM_ask_init_junction_xform $mom_sim_tool_junction
      set mom_sim_curr_jct_matrix "$mom_sim_result"
      set mom_sim_curr_jct_origin "$mom_sim_result1"

      set xval [expr [lindex $mom_sim_curr_jct_origin 0] - $mom_sim_tool_x_offset]
      set yval [expr [lindex $mom_sim_curr_jct_origin 1] - $mom_sim_tool_y_offset]
      set zval [expr [lindex $mom_sim_curr_jct_origin 2] - $mom_sim_tool_z_offset]

      set mom_sim_curr_jct_origin [list $xval $yval $zval]

      set mom_sim_tool_junction ""
      VNC_create_tmp_jct

      set mom_sim_tool_junction "$mom_sim_current_junction"
   }
}


#=============================================================
proc PB_CMD_vnc__output_debug_msg { } {
#=============================================================
  global mom_sim_message

   PB_VNC_output_debug_msg $mom_sim_message
}


#=============================================================
proc PB_CMD_vnc__output_param_msg { } {
#=============================================================
  global mom_sim_message

  #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Use MOM_output_literal to display operator messages in NC window
  #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   MOM_output_text $mom_sim_message
  # MOM_output_literal $mom_sim_message
}


#=============================================================
proc PB_CMD_vnc__output_vnc_msg { } {
#=============================================================
  global mom_sim_message
  global mom_sim_control_out mom_sim_vnc_msg_prefix mom_sim_control_in

   set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix $mom_sim_message $mom_sim_control_in"

   PB_CMD_vnc__output_param_msg
}


#=============================================================
proc PB_CMD_vnc__pass_csys_data { } {
#=============================================================
  global mom_sim_control_out mom_sim_control_in
  global mom_sim_vnc_msg_prefix
  global mom_csys_matrix

  global mom_coordinate_system_purpose
  global mom_special_output
  global mom_kin_coordinate_system_type
  global mom_machine_csys_matrix
  global mom_sim_message


   if [info exists mom_machine_csys_matrix] {

      if [string match "MAIN" $mom_kin_coordinate_system_type] {
         for { set i 0 } { $i < 12 } { incr i } {
            set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix CSYS_MTX_$i==$mom_machine_csys_matrix($i) $mom_sim_control_in"
            PB_CMD_vnc__output_param_msg
         }
      } else {

        # MTX3_multiply mom_machine_csys_matrix mom_csys_matrix matrix

         MTX3_transpose mom_machine_csys_matrix mcs_matrix
         MTX3_multiply mcs_matrix mom_csys_matrix matrix

         set u(0) $mom_csys_matrix(9)
         set u(1) $mom_csys_matrix(10)
         set u(2) $mom_csys_matrix(11)
         MTX3_vec_multiply u mom_machine_csys_matrix w

         set matrix(9)  [expr $mom_machine_csys_matrix(9)  + $w(0)]
         set matrix(10) [expr $mom_machine_csys_matrix(10) + $w(1)]
         set matrix(11) [expr $mom_machine_csys_matrix(11) + $w(2)]

         for { set i 0 } { $i < 12 } { incr i } {
            set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix CSYS_MTX_$i==$matrix($i) $mom_sim_control_in"
            PB_CMD_vnc__output_param_msg
         }
      }
   }
}


#=============================================================
proc PB_CMD_vnc__pass_head_data { } {
#=============================================================
  global mom_sim_control_out mom_sim_control_in
  global mom_sim_vnc_msg_prefix
  global mom_sys_postname
  global CURRENT_HEAD
  global mom_sim_message
  global mom_sys_sim_post_name


   set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix HEAD_NAME==$CURRENT_HEAD $mom_sim_control_in"
   PB_CMD_vnc__output_param_msg

   set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix POST_NAME==$mom_sys_sim_post_name $mom_sim_control_in"
   PB_CMD_vnc__output_param_msg
}


#=============================================================
proc PB_CMD_vnc__pass_helix_data { } {
#=============================================================
  global mom_sim_control_out mom_sim_control_in
  global mom_sim_vnc_msg_prefix
  global mom_sim_message
  global mom_sys_helix_pitch_type
  global mom_kin_helical_arc_output_mode

   if [info exists mom_sys_helix_pitch_type] {
      set type [string toupper $mom_sys_helix_pitch_type]
      set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix HELIX_PITCH_TYPE==$type $mom_sim_control_in"
      PB_CMD_vnc__output_param_msg
   }

   if [info exists mom_kin_helical_arc_output_mode] {
      set mode $mom_kin_helical_arc_output_mode
      set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix HELIX_OUTPUT_MODE==$mode $mom_sim_control_in"
      PB_CMD_vnc__output_param_msg
   }
}


#=============================================================
proc PB_CMD_vnc__pass_helix_pitch { } {
#=============================================================
  global mom_sim_control_out mom_sim_control_in
  global mom_sim_vnc_msg_prefix
  global mom_sim_message
  global mom_helix_pitch

   if [info exists mom_helix_pitch] {
      set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix HELIX_PITCH==$mom_helix_pitch $mom_sim_control_in"
      PB_CMD_vnc__output_param_msg
   }
}


#=============================================================
proc PB_CMD_vnc__pass_msys_data { } {
#=============================================================
  global mom_sim_control_out mom_sim_control_in
  global mom_sim_vnc_msg_prefix
  global mom_msys_matrix mom_msys_origin
  global mom_sim_message


   for { set i 0 } { $i < 9 } { incr i } {
      set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix CSYS_MTX_$i==$mom_msys_matrix($i) $mom_sim_control_in"
      PB_CMD_vnc__output_param_msg
   }

   for { set j 0 } { $j < 3 } { incr j } {
      set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix CSYS_MTX_$i==$mom_msys_origin($j) $mom_sim_control_in"
      incr i
      PB_CMD_vnc__output_param_msg
   }
}


#=============================================================
proc PB_CMD_vnc__pass_nurbs_data { } {
#=============================================================
  global mom_kin_nurbs_output_type

   if [string match "BSPLINE" $mom_kin_nurbs_output_type] {

      global mom_sim_control_out mom_sim_control_in
      global mom_sim_vnc_msg_prefix
      global mom_sim_message

      global mom_nurbs_knot_count
      global mom_nurbs_knots
      global mom_nurbs_point_count
      global mom_nurbs_points
      global mom_nurbs_order

      set mom_sim_message "NURBS_ORDER==$mom_nurbs_order"
      PB_CMD_vnc__output_vnc_msg

      set mom_sim_message "NURBS_KNOT_COUNT==$mom_nurbs_knot_count"
      PB_CMD_vnc__output_vnc_msg

      for {set i 0} {$i < $mom_nurbs_knot_count} {incr i} {
         set mom_sim_message "NURBS_KNOTS($i)==$mom_nurbs_knots($i)"
         PB_CMD_vnc__output_vnc_msg
      }

      set mom_sim_message "NURBS_POINT_COUNT==$mom_nurbs_point_count"
      PB_CMD_vnc__output_vnc_msg
 
      for {set i 0} {$i < $mom_nurbs_point_count} {incr i} {
         set mom_sim_message "NURBS_POINTS($i,X)==$mom_nurbs_points($i,0)"
         PB_CMD_vnc__output_vnc_msg
         set mom_sim_message "NURBS_POINTS($i,Y)==$mom_nurbs_points($i,1)"
         PB_CMD_vnc__output_vnc_msg
         set mom_sim_message "NURBS_POINTS($i,Z)==$mom_nurbs_points($i,2)"
         PB_CMD_vnc__output_vnc_msg
      }
   }
}


#=============================================================
proc PB_CMD_vnc__pass_nurbs_start { } {
#=============================================================
  global mom_kin_nurbs_output_type

   if [string match "BSPLINE" $mom_kin_nurbs_output_type] {

      global mom_sim_control_out mom_sim_control_in
      global mom_sim_vnc_msg_prefix
      global mom_sim_message

      set mom_sim_message "NURBS_START==1"
      PB_CMD_vnc__output_vnc_msg
   }
}


#=============================================================
proc PB_CMD_vnc__pass_spindle_data { } {
#=============================================================
  global mom_sim_control_out mom_sim_control_in
  global mom_sim_vnc_msg_prefix
  global mom_spindle_direction mom_spindle_mode
  global mom_spindle_speed mom_spindle_maximum_rpm
  global mom_sim_message


   set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix SPINDLE_DIRECTION==$mom_spindle_direction $mom_sim_control_in"
   PB_CMD_vnc__output_param_msg

   set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix SPINDLE_MODE==$mom_spindle_mode $mom_sim_control_in"
   PB_CMD_vnc__output_param_msg

   set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix SPINDLE_SPEED==$mom_spindle_speed $mom_sim_control_in"
   PB_CMD_vnc__output_param_msg

   set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix SPINDLE_MAX_RPM==$mom_spindle_maximum_rpm $mom_sim_control_in"
   PB_CMD_vnc__output_param_msg
}


#=============================================================
proc PB_CMD_vnc__pass_tool_data { } {
#=============================================================
  global mom_sim_control_out mom_sim_control_in
  global mom_sim_vnc_msg_prefix
  global mom_tool_name
  global mom_tool_offset_defined mom_tool_offset
  global mom_sim_message


   if { ![info exists mom_tool_offset(0)] } { set mom_tool_offset(0) 0 }
   if { ![info exists mom_tool_offset(1)] } { set mom_tool_offset(1) 0 }
   if { ![info exists mom_tool_offset(2)] } { set mom_tool_offset(2) 0 }

   set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix TOOL_NAME==$mom_tool_name $mom_sim_control_in"
   PB_CMD_vnc__output_param_msg

   set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix TOOL_OFFSET==$mom_tool_offset_defined $mom_sim_control_in"
   PB_CMD_vnc__output_param_msg

   set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix TOOL_X_OFF==$mom_tool_offset(0) $mom_sim_control_in"
   PB_CMD_vnc__output_param_msg

   set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix TOOL_Y_OFF==$mom_tool_offset(1) $mom_sim_control_in"
   PB_CMD_vnc__output_param_msg

   set mom_sim_message "$mom_sim_control_out$mom_sim_vnc_msg_prefix TOOL_Z_OFF==$mom_tool_offset(2) $mom_sim_control_in"
   PB_CMD_vnc__output_param_msg
}


#=============================================================
proc PB_CMD_vnc__pause { } {
#=============================================================
# This command enables you to pause the UG/ISV processing.
#
  VNC_pause
}


#=============================================================
proc PB_CMD_vnc__rapid_move { } {
#=============================================================
  global mom_sim_lg_axis mom_sim_pos
  global mom_sim_nc_register
  global mom_sim_num_machine_axes

  global mom_sim_prev_pos


   set pattern 0

   set coord_list [list]

   if [expr $mom_sim_pos(0) != $mom_sim_prev_pos(0)] {
      lappend coord_list $mom_sim_lg_axis(X)  $mom_sim_pos(0)
      set pattern [expr $pattern + 10000]
   }
   if [expr $mom_sim_pos(1) != $mom_sim_prev_pos(1)] {
      lappend coord_list $mom_sim_lg_axis(Y)  $mom_sim_pos(1)
      set pattern [expr $pattern + 1000]
   }
   if [expr $mom_sim_pos(2) != $mom_sim_prev_pos(2)] {
      lappend coord_list $mom_sim_lg_axis(Z)  $mom_sim_pos(2)
      set pattern [expr $pattern + 100]
   }
   if [expr $mom_sim_pos(3) != $mom_sim_prev_pos(3)] {
      lappend coord_list $mom_sim_lg_axis(4)  $mom_sim_pos(3)
      set pattern [expr $pattern + 10]
   }
   if [expr $mom_sim_pos(4) != $mom_sim_prev_pos(4)] {
      lappend coord_list $mom_sim_lg_axis(5)  $mom_sim_pos(4)
      set pattern [expr $pattern + 1]
   }

  # Disable linear axes for the 1st 5th-axis ratary move
  # right after CSYS_ROTATION is set.
   if { [info exists mom_sim_nc_register(CSYS_ROTATION)] && [expr $mom_sim_nc_register(CSYS_ROTATION) == 1] } {

      if [expr $pattern == 1] {
         global mom_sim_mt_axis

         SIM_move_rotary_axis 0 $mom_sim_mt_axis(5) $mom_sim_pos(4)
         SIM_update

        # Keep track of current position
         if { ![catch {set ref_pt [SIM_ask_last_position_zcs]} ] } {
            set mom_sim_pos(0) [lindex $ref_pt 0]
            set mom_sim_pos(1) [lindex $ref_pt 1]
            set mom_sim_pos(2) [lindex $ref_pt 2]
         }

         set pattern 0
      }

      incr mom_sim_nc_register(CSYS_ROTATION)
   }

   if [expr $pattern > 0] {
      eval VNC_move_linear_zcs RAPID $coord_list
   }

   set mom_sim_nc_register(LAST_X) $mom_sim_pos(0)
   set mom_sim_nc_register(LAST_Y) $mom_sim_pos(1)
   set mom_sim_nc_register(LAST_Z) $mom_sim_pos(2)
}


#=============================================================
proc PB_CMD_vnc__reload_kinematics { } {
#=============================================================
  global mom_sim_mt_axis
  global sim_prev_zcs
  global mom_sim_num_machine_axes mom_sim_advanced_kinematic_jct


   set current_zcs "$mom_sim_advanced_kinematic_jct"


  # Set kinematics based on the machine tool kinematics model
  # current_zcs represents the MCS used with this machine

   if { [VNC_machine_tool_model_exists]  &&  $sim_prev_zcs != $current_zcs } {

      if { $mom_sim_num_machine_axes == 4 } {

         VNC_set_post_kinematics $current_zcs $mom_sim_mt_axis(X)  $mom_sim_mt_axis(Y)  $mom_sim_mt_axis(Z)  $mom_sim_mt_axis(4)
      } elseif { $mom_sim_num_machine_axes == 5 } {

         VNC_set_post_kinematics $current_zcs $mom_sim_mt_axis(X)  $mom_sim_mt_axis(Y)  $mom_sim_mt_axis(Z)  $mom_sim_mt_axis(4)  $mom_sim_mt_axis(5)
      }

      set sim_prev_zcs $current_zcs
   }


#>>>>> Advanced kinematics
#   set uf_library "ugpadvkins.dll"
#   MOM_run_user_function [MOM_ask_env_var UGII_CAM_AUXILIARY_DIR]$uf_library ufusr


   MOM_reload_kinematics
}


#=============================================================
proc PB_CMD_vnc__reset_controller { } {
#=============================================================
  global mom_sim_nc_register


  # Initialize controller state
   set mom_sim_nc_register(MOTION)         RAPID
   set mom_sim_nc_register(UNIT)           $mom_sim_nc_register(DEF_UNIT)
   set mom_sim_nc_register(INPUT)          $mom_sim_nc_register(DEF_INPUT)
   set mom_sim_nc_register(STROKE_LIMIT)   ON
   set mom_sim_nc_register(CUTCOM)         OFF
   set mom_sim_nc_register(TL_ADJUST)      OFF
   set mom_sim_nc_register(SCALE)          OFF
   set mom_sim_nc_register(WCS)            0
   set mom_sim_nc_register(MACRO_MODAL)    OFF
   set mom_sim_nc_register(WCS_ROTATE)     OFF
   set mom_sim_nc_register(CYCLE)          OFF
   set mom_sim_nc_register(CYCLE_RETURN)   INIT_LEVEL
   set mom_sim_nc_register(RETURN_HOME)    0
   set mom_sim_nc_register(FROM_HOME)      0
}


#=============================================================
proc PB_CMD_vnc__reset_coordinate { } {
#=============================================================
  global mom_sim_nc_register
  global mom_sim_offset_pt
  global mom_sim_motion_begun
  global mom_sim_prev_pos


   if [string match "MILL" $mom_sim_nc_register(MACHINE_MODE)] {


global mom_sim_message
set mom_sim_message " Local Offset before : $mom_sim_nc_register(LOCAL_OFFSET)"
PB_CMD_vnc__send_message


if [info exists mom_sim_offset_pt] {
   set mom_sim_message "   offset pt : [array get mom_sim_offset_pt]"
   PB_CMD_vnc__send_message
}


      for {set i 0} {$i < 5} {incr i} {

        # Only change the components that exist.
         if [info exists mom_sim_offset_pt($i)] {
           #---------------------------------------------------
           # Do not change offset when the motion hasn't begun,
           # since mom_sim_prev_pos may not be reliable.
           #---------------------------------------------------
            if $mom_sim_motion_begun {
               set offset [expr $mom_sim_offset_pt($i) - $mom_sim_prev_pos($i)]
               set mom_sim_nc_register(LOCAL_OFFSET)  [lreplace $mom_sim_nc_register(LOCAL_OFFSET) $i $i $offset]
            }
         }
      }

set mom_sim_message " Local Offset after : $mom_sim_nc_register(LOCAL_OFFSET)"
PB_CMD_vnc__send_message

   }
}


#=============================================================
proc PB_CMD_vnc__return_home { } {
#=============================================================
  global mom_sim_nc_register
  global mom_sim_lg_axis
  global mom_sim_num_machine_axes
  global mom_sim_prev_pos


  # We'll simulate the subspindle movements with different codes.
   if $mom_sim_nc_register(CROSS_MACHINING) {
return
   }


  # Fetch specified components of the intermediate pt.
   set coord_list [list]
   if [info exists mom_sim_nc_register(REF_INT_PT_X)] {
      if [expr $mom_sim_nc_register(REF_INT_PT_X) != $mom_sim_prev_pos(0)] {
         lappend coord_list $mom_sim_lg_axis(X)  $mom_sim_nc_register(REF_INT_PT_X)
      }
   }
   if [info exists mom_sim_nc_register(REF_INT_PT_Y)] {
      if [expr $mom_sim_nc_register(REF_INT_PT_Y) != $mom_sim_prev_pos(1)] {
         lappend coord_list $mom_sim_lg_axis(Y)  $mom_sim_nc_register(REF_INT_PT_Y)
      }
   }
   if [info exists mom_sim_nc_register(REF_INT_PT_Z)] {
      if [expr $mom_sim_nc_register(REF_INT_PT_Z) != $mom_sim_prev_pos(2)] {
         lappend coord_list $mom_sim_lg_axis(Z)  $mom_sim_nc_register(REF_INT_PT_Z)
      }
   }

   if { $mom_sim_num_machine_axes > 3 } {
      if [info exists mom_sim_nc_register(REF_INT_PT_4)] {
         if [expr $mom_sim_nc_register(REF_INT_PT_4) != $mom_sim_prev_pos(3)] {
            lappend coord_list $mom_sim_lg_axis(4)  $mom_sim_nc_register(REF_INT_PT_4)
         }
      }
   }

   if { $mom_sim_num_machine_axes > 4 } {
      if [info exists mom_sim_nc_register(REF_INT_PT_5)] {
         if [expr $mom_sim_nc_register(REF_INT_PT_5) != $mom_sim_prev_pos(4)] {
            lappend coord_list $mom_sim_lg_axis(5)  $mom_sim_nc_register(REF_INT_PT_5)
         }
      }
   }


global mom_sim_message
set mom_sim_message "INT PT >$coord_list<"
PB_CMD_vnc__send_message


  # Position tool to intermediate pt, if specified.
   if { [llength $coord_list] > 0 } {
      eval VNC_move_linear_zcs RAPID $coord_list
   }

  # Position spindle (not tool) to reference pt
   PB_CMD_vnc__send_dogs_home


       #<07-20-04 gsl> Intermediate pt is saved for subsequent FROM_HOME move
       #               then discarded after use.
        #--------------------------------------------------
        # Initialize previous Intermediate Reference Point.
        #--------------------------------------------------
         if [info exists mom_sim_nc_register(REF_INT_PT_X_PREV)] {
            unset mom_sim_nc_register(REF_INT_PT_X_PREV)
         }
         if [info exists mom_sim_nc_register(REF_INT_PT_Y_PREV)] {
            unset mom_sim_nc_register(REF_INT_PT_Y_PREV)
         }
         if [info exists mom_sim_nc_register(REF_INT_PT_Z_PREV)] {
            unset mom_sim_nc_register(REF_INT_PT_Z_PREV)
         }
         if [info exists mom_sim_nc_register(REF_INT_PT_4_PREV)] {
            unset mom_sim_nc_register(REF_INT_PT_4_PREV)
         }
         if [info exists mom_sim_nc_register(REF_INT_PT_5_PREV)] {
            unset mom_sim_nc_register(REF_INT_PT_5_PREV)
         }

        #-----------------------------------------
        # Initialize Intermediate Reference Point.
        #-----------------------------------------
         if [info exists mom_sim_nc_register(REF_INT_PT_X)] {
            set mom_sim_nc_register(REF_INT_PT_X_PREV) $mom_sim_nc_register(REF_INT_PT_X)
            unset mom_sim_nc_register(REF_INT_PT_X)
         }
         if [info exists mom_sim_nc_register(REF_INT_PT_Y)] {
            set mom_sim_nc_register(REF_INT_PT_Y_PREV) $mom_sim_nc_register(REF_INT_PT_Y)
            unset mom_sim_nc_register(REF_INT_PT_Y)
         }
         if [info exists mom_sim_nc_register(REF_INT_PT_Z)] {
            set mom_sim_nc_register(REF_INT_PT_Z_PREV) $mom_sim_nc_register(REF_INT_PT_Z)
            unset mom_sim_nc_register(REF_INT_PT_Z)
         }
         if [info exists mom_sim_nc_register(REF_INT_PT_4)] {
            set mom_sim_nc_register(REF_INT_PT_4_PREV) $mom_sim_nc_register(REF_INT_PT_4)
            unset mom_sim_nc_register(REF_INT_PT_4)
         }
         if [info exists mom_sim_nc_register(REF_INT_PT_5)] {
            set mom_sim_nc_register(REF_INT_PT_5_PREV) $mom_sim_nc_register(REF_INT_PT_5)
            unset mom_sim_nc_register(REF_INT_PT_5)
         }

  # Reset controller
   VNC_reset_controller
}


#=============================================================
proc PB_CMD_vnc__rewind_stop_program { } {
#=============================================================
   PB_CMD_vnc__send_dogs_home
   VNC_reset_controller
}


#=============================================================
proc PB_CMD_vnc__send_dogs_home { } {
#=============================================================
  global mom_sim_mt_axis
  global mom_sim_num_machine_axes
  global mom_sim_nc_register
  global mom_sim_result1


  # Determine the axes that need to move per intermediate pt specified.
   set move_X 0
   set move_Y 0
   set move_Z 0
   set move_4 0
   set move_5 0

   SIM_ask_nc_axes_of_mtool

   if [info exists mom_sim_nc_register(REF_INT_PT_X)] {
      if { [info exists mom_sim_mt_axis(X)] } {
         if { [lsearch $mom_sim_result1 "$mom_sim_mt_axis(X)"] >= 0 } {
            set move_X 1
         }
      }
   }
   if [info exists mom_sim_nc_register(REF_INT_PT_Y)] {
      if { [info exists mom_sim_mt_axis(Y)] } {
         if { [lsearch $mom_sim_result1 "$mom_sim_mt_axis(Y)"] >= 0 } {
            set move_Y 1
         }
      }
   }
   if [info exists mom_sim_nc_register(REF_INT_PT_Z)] {
      if { [info exists mom_sim_mt_axis(Z)] } {
         if { [lsearch $mom_sim_result1 "$mom_sim_mt_axis(Z)"] >= 0 } {
            set move_Z 1
         }
      }
   }

   if { $mom_sim_num_machine_axes > 3 } {
      if [info exists mom_sim_nc_register(REF_INT_PT_4)] {
         set move_4 1
      }
   }

   if { $mom_sim_num_machine_axes > 4 } {
      if [info exists mom_sim_nc_register(REF_INT_PT_5)] {
         set move_5 1
      }
   }


  # Establish machine home position in MTCS for the 1st time.
   global mom_sim_pos_mtcs
   global mom_sim_lg_axis
   global mom_sim_spindle_jct mom_sim_current_junction


#<08-10-04 gsl>
   global mom_sim_pivot_jct
   if [info exists mom_sim_pivot_jct] {
      SIM_set_current_ref_junction $mom_sim_pivot_jct
   } else {
      SIM_set_current_ref_junction $mom_sim_spindle_jct
   }


   if { ![info exists mom_sim_pos_mtcs(0)] } {
      if $move_X {
         SIM_move_linear_axis 0 $mom_sim_mt_axis(X) [lindex $mom_sim_nc_register(REF_PT) 0]
      }
      set ref_pt [SIM_ask_last_position_mtcs]
      set mom_sim_pos_mtcs(0) [lindex $ref_pt 0]
   }
   if { ![info exists mom_sim_pos_mtcs(1)] } {
      if $move_Y {
         SIM_move_linear_axis 0 $mom_sim_mt_axis(Y) [lindex $mom_sim_nc_register(REF_PT) 1]
      }
      set ref_pt [SIM_ask_last_position_mtcs]
      set mom_sim_pos_mtcs(1) [lindex $ref_pt 1]
   }
   if { ![info exists mom_sim_pos_mtcs(2)] } {
      if $move_Z {
         SIM_move_linear_axis 0 $mom_sim_mt_axis(Z) [lindex $mom_sim_nc_register(REF_PT) 2]
      }
      set ref_pt [SIM_ask_last_position_mtcs]
      set mom_sim_pos_mtcs(2) [lindex $ref_pt 2]
   }

   set coord_list [list]
   if $move_X {
      lappend coord_list $mom_sim_lg_axis(X)  $mom_sim_pos_mtcs(0)
   }
   if $move_Y {
      lappend coord_list $mom_sim_lg_axis(Y)  $mom_sim_pos_mtcs(1)
   }
   if $move_Z {
      lappend coord_list $mom_sim_lg_axis(Z)  $mom_sim_pos_mtcs(2)
   }

#<08-10-04 gsl>
#   SIM_set_current_ref_junction $mom_sim_spindle_jct
   eval SIM_move_linear_mtcs $coord_list


global mom_sim_message
set mom_sim_message "REF PT >$mom_sim_nc_register(REF_PT)<"
PB_CMD_vnc__send_message


   global mom_sim_pos

   if [expr $mom_sim_num_machine_axes > 3] {
      if $move_4 {
         SIM_move_rotary_axis 5 $mom_sim_mt_axis(4) 0
         set mom_sim_pos(3) 0
      }
   }

   if [expr $mom_sim_num_machine_axes > 4] {
      if $move_5 {
         SIM_move_rotary_axis 5 $mom_sim_mt_axis(5) 0
         set mom_sim_pos(4) 0
      }
   }
 
   SIM_update

   SIM_set_current_ref_junction $mom_sim_current_junction

  # Keep track of current position
   if { ![catch {set ref_pt [SIM_ask_last_position_zcs]} ] } {
      set mom_sim_pos(0) [lindex $ref_pt 0]
      set mom_sim_pos(1) [lindex $ref_pt 1]
      set mom_sim_pos(2) [lindex $ref_pt 2]
   }

  # Zero Y for lathe mode
   if [string match "TURN" $mom_sim_nc_register(MACHINE_MODE)] {
      set mom_sim_pos(1) 0
   }

  # Zero all other axes
   set mom_sim_pos(5) 0
   set mom_sim_pos(6) 0
   set mom_sim_pos(7) 0
}


#=============================================================
proc PB_CMD_vnc__send_message { } {
#=============================================================
# Comment the "return" statement below to output debug messages to ISV dialog.
return

   global mom_sim_message

   PB_VNC_send_message "$mom_sim_message"
}


#=============================================================
proc PB_CMD_vnc__set_feed { } {
#=============================================================
  global mom_sim_nc_register
  global mom_sim_rapid_feed_rate

   if [expr $mom_sim_nc_register(F) == 0.0] {
      set mom_sim_nc_register(F) $mom_sim_rapid_feed_rate
   }

   SIM_set_feed $mom_sim_nc_register(F) $mom_sim_nc_register(FEED_MODE)
}


#=============================================================
proc PB_CMD_vnc__set_feedrate_mode { } {
#=============================================================
  global mom_sim_cutting_mode

   if [info exists mom_sim_cutting_mode] {
      set cutting_mode $mom_sim_cutting_mode
   } else {
      set cutting_mode "RAPID"
   }


  global mom_feed_rate
  global sim_feedrate_mode

   SIM_set_cutting_mode $cutting_mode

   set sim_feedrate_mode $cutting_mode

   if { $cutting_mode == "RAPID" } {
     # we need here the actual rapid speed. meanwhile we use what we have
      SIM_set_feed $mom_feed_rate [VNC_ask_feedrate_mode]
   } else {
      SIM_set_feed $mom_feed_rate [VNC_ask_feedrate_mode]
   }
}


#=============================================================
proc PB_CMD_vnc__set_kinematics { } {
#=============================================================
   PB_VNC_set_kinematics
}


#=============================================================
proc PB_CMD_vnc__set_nc_definitions { } {
#=============================================================
   PB_CMD_vnc____set_nc_definitions

  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Define more codes to be processed for a subordinate VNC.
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++
}


#=============================================================
proc PB_CMD_vnc__set_param_per_msg { } {
#=============================================================
  global mom_sim_msg_key mom_sim_msg_word


PB_VNC_output_debug_msg "VNC_MSG - key:$mom_sim_msg_key  word:$mom_sim_msg_word"


   switch $mom_sim_msg_key {

      "HELIX_PITCH" {
         global mom_sim_PI
         global mom_sim_helix_pitch
         set mom_sim_helix_pitch "[expr $mom_sim_msg_word / (2 * $mom_sim_PI)]"
         global mom_sim_PI
      }

      "HELIX_PITCH_TYPE" {
         global mom_sim_helix_pitch_type
         set mom_sim_helix_pitch_type "$mom_sim_msg_word"
      }

      "HELIX_OUTPUT_MODE" {
         global mom_sim_helix_output_mode
         set mom_sim_helix_output_mode "$mom_sim_msg_word"
      }

      "SPINDLE_DIRECTION" {
         global mom_sim_spindle_direction
         set mom_sim_spindle_direction "$mom_sim_msg_word"
       }

      "SPINDLE_MODE" {
         global mom_sim_spindle_mode
         switch $mom_sim_msg_word {
            RPM { set mom_sim_msg_word REV_PER_MIN }
         }
         set mom_sim_spindle_mode $mom_sim_msg_word
       }

      "SPINDLE_SPEED" {
         global mom_sim_spindle_speed mom_sim_spindle_mode
         set mom_sim_spindle_speed "$mom_sim_msg_word"

         SIM_set_speed $mom_sim_spindle_speed $mom_sim_spindle_mode
       }

      "SPINDLE_MAX_RPM" {
         global mom_sim_spindle_max_rpm
         set mom_sim_spindle_max_rpm "$mom_sim_msg_word"
       }

      "TOOL_NAME" {
         global mom_sim_ug_tool_name mom_sim_tool_change
         set mom_sim_ug_tool_name "$mom_sim_msg_word"
         set mom_sim_tool_change 1
       }

      "TOOL_OFFSET" {
         global mom_sim_tool_offset_used
         set mom_sim_tool_offset_used "$mom_sim_msg_word"
      }

      "TOOL_X_OFF" {
         global mom_sim_tool_offset
         set mom_sim_tool_offset(0) "$mom_sim_msg_word"
      }

      "TOOL_Y_OFF" {
         global mom_sim_tool_offset
         set mom_sim_tool_offset(1) "$mom_sim_msg_word"
      }

      "TOOL_Z_OFF" {
         global mom_sim_tool_offset
         set mom_sim_tool_offset(2) "$mom_sim_msg_word"
      }

      "HEAD_NAME" {
         global mom_sim_machine_head
         set mom_sim_machine_head "$mom_sim_msg_word"
      }

      "POST_NAME" {
         global mom_sim_post_name
         set mom_sim_post_name "$mom_sim_msg_word"

         global mom_sim_vnc_handler_loaded

         set new_vnc_file [string trim ${mom_sim_post_name}_vnc.tcl]
         set new_vnc_file [join [split $new_vnc_file \\] /]

         if { [string compare [string trim $mom_sim_vnc_handler_loaded] $new_vnc_file] } {
            uplevel #0 {
               source ${mom_sim_post_name}_vnc.tcl
            }

            PB_CMD_vnc__init_sim_vars
         }
      }

      "CYCLE_SPINDLE_AXIS" {
         global mom_sim_cycle_spindle_axis
         set mom_sim_cycle_spindle_axis "$mom_sim_msg_word"
      }

      "FIXTURE_OFFSET" {
      }

      default {
      }
   }


  # Grab NURBS
   global mom_sim_nurbs_start
   global mom_sim_nurbs_order
   global mom_sim_nurbs_knot_count
   global mom_sim_nurbs_knots
   global mom_sim_nurbs_point_count
   global mom_sim_nurbs_points
   global mom_sim_nurbs_point_end_index

   if [string match "NURBS_START" $mom_sim_msg_key] {
      set mom_sim_nurbs_start 0
   }

   if [string match "NURBS_ORDER" $mom_sim_msg_key] {
      set mom_sim_nurbs_order $mom_sim_msg_word
   }

   if [string match "NURBS_KNOT_COUNT" $mom_sim_msg_key] {
      set mom_sim_nurbs_knot_count $mom_sim_msg_word
      set mom_sim_nurbs_knots [list]
   }

   if [string match "NURBS_POINT_COUNT" $mom_sim_msg_key] {
      set mom_sim_nurbs_point_count $mom_sim_msg_word
      set mom_sim_nurbs_points [list]
      set mom_sim_nurbs_point_end_index [expr $mom_sim_nurbs_point_count - 1]
   }

   if [string match "NURBS_KNOTS*" $mom_sim_msg_key] {
      lappend mom_sim_nurbs_knots $mom_sim_msg_word
   }

  # Info of control points is the last set of parameters passed from a NURBS event.
  # (See PB_CMD_vnc__pass_nurbs_data)
  #
   if [string match "NURBS_POINTS*" $mom_sim_msg_key] {
      lappend mom_sim_nurbs_points $mom_sim_msg_word

     # When the control points list is completed, set off simulation.
      if [string match "*($mom_sim_nurbs_point_end_index,Z)*" $mom_sim_msg_key] {
         global mom_sim_nc_register
         set mom_sim_nc_register(MOTION) NURBS

         set mom_sim_nurbs_start 1
         PB_CMD_vnc__sim_motion

         unset mom_sim_nurbs_start
      }
   }


  # Grab CSYS_MTX_'s
   global mom_sim_csys_matrix mom_sim_csys_set

   if $mom_sim_csys_set {
return
   }

   if [string match "CSYS_MTX_*" $mom_sim_msg_key] {
      set tokens [split $mom_sim_msg_key _]
      set idx [lindex $tokens 2]

      set mom_sim_csys_matrix($idx) "$mom_sim_msg_word"

     # When csys is completely defined, map and set ZCS junction for simulation.
      if { $idx == 11 } {
         PB_CMD_vnc__set_kinematics
         set mom_sim_csys_set 1
      }
   }
}


#=============================================================
proc PB_CMD_vnc__set_speed { } {
#=============================================================
  global mom_sim_nc_register

   SIM_set_speed $mom_sim_nc_register(S) $mom_sim_nc_register(SPINDLE_MODE)
}


#=============================================================
proc PB_CMD_vnc__sim_motion { } {
#=============================================================
  global mom_sim_nc_register
  global mom_sim_nurbs_start


  # Disable simulation when a NURBS definition starts.
   if { [info exists mom_sim_nurbs_start] && !$mom_sim_nurbs_start } {
return
   }

  # Set feed & speed
   PB_CMD_vnc__set_feed
   PB_CMD_vnc__set_speed


  # We'll simulate the sub-spindle movements with different codes.
   if $mom_sim_nc_register(CROSS_MACHINING) {
return
   }


global mom_sim_o_buffer
global mom_sim_message
set mom_sim_message "** $mom_sim_nc_register(MOTION) -- $mom_sim_o_buffer **"
PB_CMD_vnc__send_message


  # Create a temp jct for subsequent moves incase the tool is not set properly.
   VNC_create_tmp_jct

   switch $mom_sim_nc_register(MOTION) {
      "RAPID" {
         VNC_rapid_move
      }

      "LINEAR" {
         VNC_linear_move
      }

      "CIRCULAR_CLW" {
         VNC_circular_move CLW
      }

      "CIRCULAR_CCLW" {
         VNC_circular_move CCLW
      }

      "NURBS" {
         if $mom_sim_nurbs_start {
            VNC_nurbs_move
         }
      }
   }

   if [string match "CYCLE_*" $mom_sim_nc_register(MOTION)] {
      VNC_cycle_move
   }
}


#=============================================================
proc PB_CMD_vnc__sim_other_devices { } {
#=============================================================
   PB_CMD_vnc____sim_other_devices

  #++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Simulate more devices specific for a subordinate VNC.
  #++++++++++++++++++++++++++++++++++++++++++++++++++++++
}


#=============================================================
proc PB_CMD_vnc__start_of_path { } {
#=============================================================

  # Redefine MOM_helix_move & pass info to VNC.
   if { [llength [info commands "MOM_SYS_helix_move"]] == 0 } {
      if [llength [info commands "MOM_helix_move"]] {
         rename MOM_helix_move MOM_SYS_helix_move

         uplevel #0 {

            proc MOM_helix_move {} {
               PB_CMD_vnc__pass_helix_pitch
               MOM_SYS_helix_move
            }
         }

        # Fetch global pitch info
         PB_CMD_vnc__pass_helix_data
      }
   }


  # Redefine MOM_nurbs_move & pass info to VNC.
   if { [llength [info commands "MOM_SYS_nurbs_move"]] == 0 } {
      if [llength [info commands "MOM_nurbs_move"]] {

         rename MOM_nurbs_move MOM_SYS_nurbs_move

         uplevel #0 {

            proc MOM_nurbs_move {} {

               PB_CMD_vnc__pass_nurbs_start

               MOM_SYS_nurbs_move

               PB_CMD_vnc__pass_nurbs_data
            }
         }
      }
   }
}


#=============================================================
proc PB_CMD_vnc__start_of_program { } {
#=============================================================
  global mom_multi_channel_mode

   if [info exists mom_multi_channel_mode] {
     # Set up sync manager dialog
      if [llength [info commands "PB_CMD_vnc__config_sync_dialog"]] {
         PB_CMD_vnc__config_sync_dialog
      }

     # Initialize parameters for sync manager
      if [llength [info commands "PB_CMD_vnc__init_sync_manager"]] {
         PB_CMD_vnc__init_sync_manager
      }
   }

  # Mount part onto sub-spindle
   if { [llength [info commands "PB_CMD_vnc__mount_part"]] } {
      PB_CMD_vnc__mount_part
   }
}


#=============================================================
proc PB_CMD_vnc__sync { } {
#=============================================================
  global mom_sync_number
  global mom_sync_primary
  global mom_sync_affected
  global mom_carrier_name
  global mom_sim_primary_channel
  global sync_event_detected_on_channel_1
  global sync_event_detected_on_channel_2


  #MOM_output_to_listing_device "SYNC : $mom_sync_number $mom_sync_primary "
  #MOM_output_to_listing_device "SYNC : $mom_sync_affected(0) $mom_sync_affected(1) "


    SIM_synchronize $mom_sync_number $mom_sync_affected(0) $mom_sync_affected(1)


    #SIM_primary_channel $mom_sync_primary
    #set mom_sim_primary_channel $mom_sync_primary
}


#=============================================================
proc PB_CMD_vnc__tool_change { } {
#=============================================================
  global sim_prev_tool_name
  global mom_sim_result mom_sim_result1
  global mom_sim_ug_tool_name
  global mom_sim_tool_loaded
  global mom_sim_spindle_comp mom_sim_spindle_jct
  global mom_sim_lg_axis
  global mom_sim_nc_register
  global mom_sim_machine_type
  global mom_sim_tool_offset_used mom_sim_tool_offset mom_sim_tool_mount
  global mom_sim_output_reference_method
  global mom_sim_tool_junction mom_sim_current_junction
  global mom_sim_pivot_distance
  global mom_sim_tool_change

  global mom_sim_pos mom_sim_prev_pos


  # Current tool has been loaded. Do not change tool.
   if { [info exists mom_sim_tool_change] && !$mom_sim_tool_change } {
return
   }


  # Position spindle to reference point.
   PB_CMD_vnc__send_dogs_home

   set sim_tool_name ""

   if { ![string match "$sim_prev_tool_name" $mom_sim_ug_tool_name] &&  ![string match "" $mom_sim_ug_tool_name] } {

#>>>>>
#  This is where you add detailed movements such as rotating tool changer,
#  unmounting old tool and mounting new tool and/or getting next tool in ready position.
#<<<<<

      VNC_unmount_tool $sim_prev_tool_name
      SIM_mount_tool "15" "UG_NAME" "$mom_sim_ug_tool_name"  "$mom_sim_spindle_comp" "$mom_sim_spindle_jct"

      set sim_tool_name $mom_sim_result 

      if {$sim_tool_name != ""} {
         SIM_activate_tool $sim_tool_name
      }

      SIM_update 

      set sim_prev_tool_name $mom_sim_ug_tool_name

      set mom_sim_tool_loaded $sim_tool_name

     # Flag that a tool change is done.
      set mom_sim_tool_change 0

      VNC_set_ref_jct $sim_tool_name
 
      set mom_sim_tool_junction "$mom_sim_current_junction"


     #++++++++++++++++++++++++++++++++++++++++++++++++++++++
     # Create a junction per tool offsets to track N/C data.
     #++++++++++++++++++++++++++++++++++++++++++++++++++++++
      set use_tool_tip_jct 1

      if { $mom_sim_tool_offset_used || [expr $mom_sim_pivot_distance != 0.0] } {

         if { ![string match "*_wedm" $mom_sim_machine_type] } {
            if [string match "*axis*" $mom_sim_machine_type] {
               set use_tool_tip_jct 0
               set x_offset [expr -1.0 * ($mom_sim_tool_offset(2) + $mom_sim_pivot_distance)]
               set y_offset 0.0
               set z_offset 0.0

            } elseif [string match "*lathe*" $mom_sim_machine_type] {
               if [string match "TURRET_REF" $mom_sim_output_reference_method] {
                  set use_tool_tip_jct 0
                  set x_offset $mom_sim_tool_offset(0)
                  set y_offset $mom_sim_tool_offset(1)
                  set z_offset 0.0
               }
            }
         }
      }

      if { !$use_tool_tip_jct } {
         global mom_sim_tool_x_offset mom_sim_tool_y_offset mom_sim_tool_z_offset

         set mom_sim_tool_x_offset $x_offset
         set mom_sim_tool_y_offset $y_offset
         set mom_sim_tool_z_offset $z_offset

         PB_CMD_vnc__offset_tool_jct
      }

      if [string match "TURN" $mom_sim_nc_register(MACHINE_MODE)] {
         set mom_sim_pos(1) 0
      }

     # Reset mom_sim_prev_pos to disable unnecessary move
      for {set i 0} {$i < 5} {incr i} {
         set mom_sim_prev_pos($i) $mom_sim_pos($i)
      }
   }
}


#=============================================================
proc PB_CMD_vnc__unset_Y_axis { } {
#=============================================================
  global mom_sim_mt_axis


  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # For a lathe that doesn't have Y-axis defined in the model,
  # uncomment the line below and call this command in the end
  # of command PB_CMD_vnc____map_machine_tool_axes.
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # unset mom_sim_mt_axis(Y)


  # Map logical names (addresses) to physical axes (machine tool model). 
   global mom_sim_result1
   SIM_ask_nc_axes_of_mtool

   set axes_map_list [list]
   set axes_config_list [list]

   if { [lsearch $mom_sim_result1 "$mom_sim_mt_axis(X)"] >= 0 } {
      lappend axes_map_list $mom_sim_address(X,leader) $mom_sim_mt_axis(X)
      lappend axes_config_list $mom_sim_mt_axis(X)
   }

   if { [info exists mom_sim_mt_axis(Y)] } {
      if { [lsearch $mom_sim_result1 "$mom_sim_mt_axis(Y)"] >= 0 } {
         lappend axes_map_list $mom_sim_address(Y,leader) $mom_sim_mt_axis(Y)
         lappend axes_config_list $mom_sim_mt_axis(Y)
      }
   }

   if { [lsearch $mom_sim_result1 "$mom_sim_mt_axis(Z)"] >= 0 } {
      lappend axes_map_list $mom_sim_address(Z,leader) $mom_sim_mt_axis(Z)
      lappend axes_config_list $mom_sim_mt_axis(Z)
   }

   SIM_set_linear_axes_map    [concat $axes_map_list]
   SIM_set_linear_axes_config [concat $axes_config_list]
}
